#include"Eli_App.h"

int _listen_cb (const char *key, const Ecore_Config_Type type,
         const int tag, void *data);
	 
void eli_app_config_init(Eli_App * eap)
{
	char * gui;
	char * cards;
	
	/* THEME*/
	ecore_config_theme_default("/theme/gui","default");
	ecore_config_theme_preview_group_set("/theme/gui","main");
	ecore_config_listen("theme", "/theme/gui", _listen_cb, THEME, eap);
	/* CARDS */
	ecore_config_theme_default("/theme/cards","default_cards");
	ecore_config_theme_preview_group_set("/theme/cards","01");
	ecore_config_listen("cards", "/theme/cards", _listen_cb, CARDS, eap);
	/* VELOCITY */
	ecore_config_int_default("velocity", 400);
	ecore_config_listen("velocity", "velocity", _listen_cb, VELOCITY, eap);
	/* FRAME RATE */
	ecore_config_int_default("frame_rate", 100);
	ecore_config_listen("frame_rate", "frame_rate", _listen_cb, FRAME_RATE, eap);
	/* ANIMATIONS */
	ecore_config_boolean_default("/graphic/animations",1);
	ecore_config_listen("/graphic/animations", "/graphic/animations", _listen_cb, ANIMATIONS, eap);
	/* SHADOWS */
	ecore_config_boolean_default("/graphic/shadows",1);
	ecore_config_listen("/graphic/shadows", "/graphic/shadows", _listen_cb, SHADOWS, eap);
	/* GL or SOFTWARE */
	ecore_config_boolean_default("/graphic/gl",0);
	
	ecore_config_load();
	
	/* set the themes */
	gui 	= ecore_config_theme_get("/theme/gui");
	cards	= ecore_config_theme_get("/theme/cards");
	
	if(eli_app_theme_gui_exists(eap,gui)) eap->theme.gui.current = gui;
	else {
		/* take the first theme if the wanted doesnt exists */
		char * src;
		src = (char*) ecore_list_goto_first(eap->theme.gui.list);
		if(!src) fprintf( stderr, _("Elitaire Error: No theme file in %s!\n"), 
				ecore_config_theme_search_path_get());
		eap->theme.gui.current = strdup(src);
	}
	
	if(eli_app_theme_cards_exists(eap,cards)) eap->theme.cards.current = cards;
	else {
		/* take the first theme if the wanted doesnt exists */
		char * src;
		src = (char*) ecore_list_goto_first(eap->theme.cards.list);
		if(!src) fprintf( stderr, _("Elitaire Error: No cards file in %s!\n"),
				ecore_config_theme_search_path_get());
		else eap->theme.cards.current = strdup(src);
	}
}

int _listen_cb (const char *key, const Ecore_Config_Type type,
         const int tag, void *data)
{
	Eli_App * eap;
	
	eap = (Eli_App*) data;
	
	switch(tag){
		case THEME:
			if(!eap->gui) break;
			else {
				char* theme;
				theme = ecore_config_theme_get("/theme/gui");
				eli_app_theme_change(eap, theme);
			}
			break;
		case CARDS:
			if(!eap->elitaire) break;
			else {
				char* cards;
				cards = ecore_config_theme_get("/theme/cards");
				if(eap->theme.cards.current) free(eap->theme.cards.current); // free the old one
				eap->theme.cards.current = cards;
				cards = ecore_config_theme_with_path_from_name_get(eap->theme.cards.current);
				elitaire_file_set(eap->elitaire, cards);
			}
			break;
		case VELOCITY:
			int v;
			v = ecore_config_int_get("velocity");
			if(eap->gui) eli_app_velocity_set(eap,v);
			if(!eap->elitaire) break;
			else {
				elitaire_velocity_set(eap->elitaire,v);
			}
			break;
		case ANIMATIONS:
			if(!eap->elitaire) break;
			else {
				Evas_Bool b;
				b = ecore_config_boolean_get("/graphic/animations");
				elitaire_animations_set(eap->elitaire,b);
			}
			break;
		case SHADOWS:
			if(!eap->elitaire) break;
			else {
				Evas_Bool b;
				b = ecore_config_boolean_get("/graphic/shadows");
				elitaire_shadows_set(eap->elitaire,b);
			}
			break;
		case FRAME_RATE:
			if(!eap->elitaire) break;
			else {
				int fr;
				fr = ecore_config_int_get("frame_rate");
				elitaire_frame_rate_set(eap->elitaire,fr);
			}
			break;
		default: break;
	}
	
	return 0;
}
