#include"Elitaire.h"

/* *********************************************************************************************** */
/* Declarations                                       							         */
/* *********************************************************************************************** */

/* Internals */
int 		_elitaire_cols_max_calc(Elitaire * eli);
int 		_elitaire_num_of_rows_calc(Elitaire * eli); /* calculate  the muber of rows*/
// void		_elitaire_reinit(Elitaire* eli);  /*move the cards to the right place*/
Evas_Coord 	_elitaire_spacing_abs_calc(Elitaire * eli, int row);
void 		_elitaire_cards_del(Elitaire * eli);
void 		_elitaire_stacks_del(Elitaire * eli);
void 		_elitaire_force_resize(Elitaire * eli);


/* Callbacks */
void 		_elitaire_mouse_up_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
void 		_elitaire_mouse_down_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
void 		_elitaire_stack_mouse_up_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
void		_elitaire_move_cb(playingCard* pcard);
void		_elitaire_flip_cb(playingCard* pcard);
void		_elitaire_move_no_ani_cb(playingCard* pcard);
void		_elitaire_flip_no_ani_cb(playingCard* pcard);


/* smart object handlers */
Evas_Smart * _elitaire_smart_get();

void _elitaire_add(Evas_Object *o);
void _elitaire_del(Evas_Object *o);
void _elitaire_layer_set(Evas_Object *o, int l);
void _elitaire_raise(Evas_Object *o);
void _elitaire_lower(Evas_Object *o);
void _elitaire_stack_above(Evas_Object *o, Evas_Object *above);
void _elitaire_stack_below(Evas_Object *o, Evas_Object *below);
void _elitaire_move(Evas_Object *o, Evas_Coord x, Evas_Coord y);
void _elitaire_resize(Evas_Object *o, Evas_Coord w, Evas_Coord h);
void _elitaire_show(Evas_Object *o);
void _elitaire_hide(Evas_Object *o);
void _elitaire_color_set(Evas_Object *o, int r, int g, int b, int a);
void _elitaire_clip_set(Evas_Object *o, Evas_Object *clip);
void _elitaire_clip_unset(Evas_Object *o);

/* keep a global copy of this, so it only has to be created once */
static Evas_Smart *smart;

/* what kind of games can we play */
const char * elitaire_available_games[] = {
	"klondike", 
	"freecell",
	"Baker's Dozen",
	"thoughtful",
	""   		/* this is the end */
	};
/* *********************************************************************************************** */
/* external available functions							         */
/* *********************************************************************************************** */
const char ** elitaire_available_games_get()
{
	return elitaire_available_games;
}
	
Evas_Object * elitaire_new(Evas* evas, const char * game)
{
	Evas_Object *elitaire;
	Elitaire * eli;
	
	elitaire = evas_object_smart_add(evas, _elitaire_smart_get());
	
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	if(!eli) return NULL;
	
	/* choose game */
	if(!strcmp(game,"klondike")){
		eli->game = new klondike;
	}
	else if(!strcmp(game,"freecell")){
		eli->game = new freeCell;
	}
	else if(!strcmp(game,"thoughtful")){
		eli->game = new klondike(true);
	}
	else if(!strcmp(game,"Baker's Dozen")){
		eli->game = new bakersDozen;
	}
	else{
		fprintf( stderr, _("Elitaire Error: unknown game %s\n"), game);
		return NULL;
	}
	
	/* init variables */
	eli->evas = evas;
	eli->file = NULL;
	eli->velocity = 1000;
	eli->spacing = 0.1;
	eli->margin_h = 0.4;
	eli->margin_v = 0.15;
	eli->x = 0;
	eli->y = 0;
	eli->w = 0;
	eli->h = 0;
	eli->frame_rate = 80;
	eli->deal = false;
	eli->shadow = false;
	eli->animate = true;
	
	/* calculate  the longest line */
	eli->max_cols = _elitaire_cols_max_calc(eli);
	eli->num_rows = _elitaire_num_of_rows_calc(eli);
	
	/* how many stacks? */
	eli->num_stacks = eli->game->getNumStacks();
	
	/* how many cards are in the game */
	eli->num_cards = eli->game->getNumCards();
	
	/* add callbacks */
	if(eli->animate){
		eli->game->addMoveCallback(_elitaire_move_cb);
		eli->game->addFlipCallback(_elitaire_flip_cb);
	}
	else {
		eli->game->addMoveCallback(_elitaire_move_no_ani_cb);
		eli->game->addFlipCallback(_elitaire_flip_no_ani_cb);
	}
	
	/* init clip */
	eli->clip = evas_object_rectangle_add(eli->evas);
	//evas_object_color_set(eli->clip,0,0,0,200);
	
	return elitaire;
}
	
int elitaire_file_set(Evas_Object * elitaire,  char *file)
{
	
	int i, number;
	playingCard* pcard;
	Evas_Object* card;
	Elitaire * eli;
	vector<stackLook>::const_iterator iord, iend;
	Evas_Bool change_file;
	
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	if(!eli) return 0;
		
	/* for the stack images */
	iord = eli->game->getStackLooksBegin();
	iend = eli->game->getStackLooksEnd();
	
	if(eli->file) change_file = true;
	else change_file = false;
	
	if(!file) return 0;
	else eli->file = file;
	
	/* get the maximal and minimal size of the cards via edje */
	/* I expect that every card in the game has the same size*/
	card = edje_object_add(eli->evas);
	if(!edje_object_file_set(card,eli->file,"00")) return 0;
	edje_object_size_max_get(card, &eli->card_max_w, &eli->card_max_h);
	edje_object_size_min_get(card, &eli->card_min_w, &eli->card_min_h);
	evas_object_del(card);
	
	/*connect the playingCards with elitaire_cards and set the callbacks*/
	if(change_file) _elitaire_cards_del(eli);
		
	for(i=0;i < eli->num_cards;i++){
		pcard = eli->game->getCard(i,0);
		card = elitaire_card_new(eli,pcard);
		evas_object_clip_set(card,eli->clip);
		evas_object_show(card);
		pcard->data = (void*) card;
		elitaire_card_callback_add(card, EVAS_CALLBACK_MOUSE_DOWN, _elitaire_mouse_down_cb);
		elitaire_card_callback_add(card, EVAS_CALLBACK_MOUSE_UP, _elitaire_mouse_up_cb);
		if (evas_alloc_error() != EVAS_ALLOC_ERROR_NONE){
			fprintf( stderr, _("Elitaire Error: Callback registering failed!\n"));
			return 0;
		}
	}	
	
	/* give the stacks an pic */
	if(change_file) _elitaire_stacks_del(eli);
	
	eli->stacks = (Evas_Object**) calloc(eli->num_stacks,sizeof(Elitaire*));
	number = 0;
	
	while(iord != iend)
	{
		switch(*iord){
			case STACK:
			case LAST:
				eli->stacks[number] = edje_object_add(eli->evas);
				edje_object_file_set(eli->stacks[number],eli->file,"LAST");
				evas_object_event_callback_add(eli->stacks[number], EVAS_CALLBACK_MOUSE_UP, _elitaire_stack_mouse_up_cb, eli);
				evas_object_show(eli->stacks[number]);
				number++;
				break;
				
			case ALL:
				eli->stacks[number] = edje_object_add(eli->evas);
				edje_object_file_set(eli->stacks[number],eli->file,"ALL");
				evas_object_event_callback_add(eli->stacks[number], EVAS_CALLBACK_MOUSE_UP, _elitaire_stack_mouse_up_cb, eli);
				evas_object_show(eli->stacks[number]);
				number++;
				break;
			default: break;
		}
		iord++;
	}
	
	if(change_file)
	{
		_elitaire_force_resize(eli);
		_elitaire_reinit(eli);
	}
	return 1;
}

void elitaire_size_min_get(Evas_Object * elitaire, Evas_Coord *minw, Evas_Coord *minh)
{
	Elitaire *eli;
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	
	*minw = (Evas_Coord)(eli->card_min_w * ( ((float) eli->max_cols)     
		+ eli->spacing * ((float) (eli->max_cols -1.0)) + (float) (2.0 * eli->margin_h)));
	*minh = 2 * (Evas_Coord) ((float) eli->num_rows * eli->card_min_h + (float) (eli->num_rows -1.0) 
		* eli->card_min_w * eli->spacing + 2.0 * eli->margin_h * eli->card_min_h);
		
}

void elitaire_size_max_get(Evas_Object * elitaire, Evas_Coord *maxw, Evas_Coord *maxh)
{
	Elitaire *eli;
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	
	if(maxw) *maxw = (Evas_Coord) (eli->card_max_w * ( ((float) eli->max_cols)     
		+ eli->spacing * ((float) (eli->max_cols -1.0)) + (float) (2.0 * eli->margin_h)));
	if(maxh) *maxh = 2 * (Evas_Coord) ((float) eli->num_rows * eli->card_max_h + (float) (eli->num_rows -1.0) 
		* eli->card_max_w * eli->spacing + 2.0 * eli->margin_h * eli->card_max_h);
	
}
int elitaire_file_get(Evas_Object * elitaire, char **file)
{	
	Elitaire *eli;
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	
	if(eli){
		file = &eli->file;	
		return 1;
	}
	return 0;
}

int elitaire_velocity_get(Evas_Object * elitaire)
{
	Elitaire *eli;
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	
	if(eli) return eli->velocity;
	return 0;
}

void elitaire_velocity_set(Evas_Object * elitaire, int v)
{
	Elitaire *eli;
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	
	if(eli){	
		eli->velocity = v;
	}
	
}

void elitaire_frame_rate_set(Evas_Object * elitaire, int fr)
{
	Elitaire *eli;
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	
	if(eli){	
		eli->frame_rate = fr;
	}
	
}

int elitaire_frame_rate_get(Evas_Object * elitaire)
{
	Elitaire *eli;
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	
	if(eli) return eli->frame_rate;
	return 0;
}

void elitaire_animations_set(Evas_Object * elitaire, Evas_Bool on)
{
	Elitaire *eli;
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	
	
	if(eli){	
		if(eli->animate == on) return;
		eli->animate = on;
		if(eli->animate){
			eli->game->addMoveCallback(_elitaire_move_cb);
			eli->game->addFlipCallback(_elitaire_flip_cb);
		}
		else {
			eli->game->addMoveCallback(_elitaire_move_no_ani_cb);
			eli->game->addFlipCallback(_elitaire_flip_no_ani_cb);
		}
	}
	
}

Evas_Bool elitaire_animations_get(Evas_Object * elitaire)
{
	Elitaire *eli;
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	
	if(eli) return eli->animate;
	return 0;
}

void elitaire_shadows_set(Evas_Object * elitaire, Evas_Bool on)
{
	Elitaire *eli;
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	
	if(eli){	
		eli->shadow = on;
	}
}

Evas_Bool elitaire_shadows_get(Evas_Object * elitaire)
{
	Elitaire *eli;
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	
	if(eli) return eli->shadow;
	return 0;
}

void elitaire_deal(Evas_Object * elitaire)
{
	
	Elitaire *eli;
	
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	if(eli){
		if(!eli->deal){
			eli->game->deal();
			eli->deal = true;
		}
	}
}

void elitaire_win_callback_add(Evas_Object * elitaire, void (*func)(int win, void* data), void* data)
{
	Elitaire *eli;
	
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	if(eli){
		eli->game->addWinCallback(func,data);
	}
}

void elitaire_points_callback_add(Evas_Object * elitaire, void (*func)(float, pointsType, void*), void* data)
{
	Elitaire *eli;
	
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	if(eli){
		eli->game->addPointsCallback(func,data);
	}
}

int elitaire_clicked_stack_get(Elitaire* eli, Evas_Coord x, Evas_Coord y)
{
	
	vector<stackLook>::const_iterator iord = eli->game->getStackLooksBegin();
	vector<stackLook>::const_iterator iend = eli->game->getStackLooksEnd();
		
	int number = 0, line = 1;
	/* rem_x,rem_y are for remembering the coords of the start of a line or something similar */
	//Evas_Coord rem_x , rem_y; 
	
	Evas_Coord rel_x = eli->x + (Evas_Coord) (eli->card_w * eli->margin_h);
	Evas_Coord rel_y = eli->y + (Evas_Coord) (eli->card_h * eli->margin_v);
	Evas_Coord rel_y2, rel_x2, offset;
	Evas_Coord spacing;
	
	offset = _elitaire_card_offset_get(eli);
	
	rel_x2 = rel_x;
	spacing = _elitaire_spacing_abs_calc(eli,line);
	
	
	while(iord != iend)
	{
		switch(*iord){
			case STACK:
			case LAST:
				
				if( 0 < (x - rel_x2) && eli->card_w > (x - rel_x2) && 0 < (y - rel_y) && eli->card_h > (y - rel_y)){
					return number;
				}
				rel_x2 += eli->card_w + spacing;
				number++;
				break;
				
			case SEPERATOR:
				
				rel_x2 += eli->card_w + spacing;
				break;
			case ENDL:
				
				rel_y += eli->card_h + (Evas_Coord)(eli->spacing * eli->card_w);
				line++;
				spacing = _elitaire_spacing_abs_calc(eli,line);
				rel_x2 = rel_x;
				
				break;
			case ALL:
				
				rel_y2 = rel_y + offset * (eli->game->getStackEnd(number) - eli->game->getStackBegin(number));
					
				if( 0 < (x - rel_x2) && eli->card_w > (x - rel_x2) && 0 < (y - rel_y2) && eli->card_h > (y - rel_y2)){
					return number;
				}
	
				rel_x2 += eli->card_w + spacing;
				number++;
				
				break;
		}
		iord++;
	}
	return -1;
}

void elitaire_stack_layer_reinit(Elitaire* eli, cardStack* stack)
{
	vector<playingCard*>::const_iterator itBegin,itEnd;
	int i=1;
	Evas_Object* card;
	
	itBegin = stack->begin();
	itEnd = stack->end();
	
	while(itBegin != itEnd){
		if(card = (Evas_Object*) (*itBegin)->data){
			evas_object_layer_set(card,i);
		}
		itBegin++;
		i++;
	}
}

void elitaire_undo(Evas_Object* elitaire)
{
	Elitaire *eli;
	
	eli = (Elitaire*) evas_object_smart_data_get(elitaire);
	if(eli){
		eli->game->undo();
	}
}
Evas_Coord _elitaire_card_offset_get(Elitaire * eli){
	return eli->card_h/5 ;
}

Evas_Coord _elitaire_card_offset_calc(Elitaire * eli, Evas_Coord h){
	return h/5;
}
/* ********************************************************************************************************* */
/* Internals                                                              	 	 	 	 			          */
/* ********************************************************************************************************* */
int _elitaire_cols_max_calc(Elitaire * eli){ /* calculate  the longest line */
	
	int i, max_i;
	vector<stackLook>::const_iterator it_begin = eli->game->getStackLooksBegin();
	vector<stackLook>::const_iterator it_end = eli->game->getStackLooksEnd();
	
	i = 0; max_i = 0;
	
	while(it_begin != it_end){
		if(*(it_begin++) == ENDL){
			max_i = (max_i < i) ? i : max_i;
			i = 0;
		}
		else i++;
	}
	return max_i;
}

/* calculate  the muber of rows*/
int _elitaire_num_of_rows_calc(Elitaire * eli){ 
	
	int num;
	vector<stackLook>::const_iterator it_begin = eli->game->getStackLooksBegin();
	vector<stackLook>::const_iterator it_end = eli->game->getStackLooksEnd();
	
	num = 1;
	
	while(it_begin != it_end){
		if(*(it_begin++) == ENDL) num++;
	}
	
	return num;
}

/* calculate  the length of an row */
Evas_Coord _elitaire_spacing_abs_calc(Elitaire * eli, int row){  
	
	int i, irow;
	vector<stackLook>::const_iterator it_begin,it_end;
	
	it_begin = eli->game->getStackLooksBegin();
	it_end   = eli->game->getStackLooksEnd();
	i = 0; irow = 1;
	
	while(it_begin != it_end){
		if(*(it_begin++) == ENDL){
			if(irow == row) break;
			i = 0;
			irow++;
		}
		else i++;
	}
	return (Evas_Coord) (eli->w - 2 * eli->margin_h * eli->card_w - i * eli->card_w) / (i - 1);
} 

/* this function convert an integer val int to an char array buf 
    it use the base=10 , be carful, it doesn"t  alloc any memory so the buf array should have 
   (length+1) elements */

char* itoa(int val, char* buf, int length){
        
        buf[length--] = 0;
        
        do{
                buf[length--] = "0123456789"[val % 10];
                val /= 10;
        }while(length+1);
        return buf;
}

void _elitaire_reinit(Elitaire* eli)
{
	vector<playingCard*>::const_iterator itBegin,itEnd;
	vector<stackLook>::const_iterator iord = eli->game->getStackLooksBegin();
	vector<stackLook>::const_iterator iend = eli->game->getStackLooksEnd();
	
	Evas_Object * card = NULL;
	Evas_Object * remember_card = NULL;
	
	int number = 0, line = 1, i = 1;
	Evas_Coord rel_x = eli->x + (Evas_Coord) (eli->card_w * eli->margin_h);
	Evas_Coord rel_y = eli->y + (Evas_Coord) (eli->card_h * eli->margin_v);
	Evas_Coord rel_y2, rel_x2, offset;
	Evas_Coord spacing;
	
	offset = _elitaire_card_offset_get(eli);
	
	rel_x2 = rel_x;
	spacing = _elitaire_spacing_abs_calc(eli,line);
	
	
	while(iord != iend)
	{
		switch(*iord){
			case STACK:
			case LAST:
				itBegin = eli->game->getStackBegin(number);
				itEnd = eli->game->getStackEnd(number);
				evas_object_move(eli->stacks[number],rel_x2,rel_y);
				
				while(itBegin != itEnd){
					if(card = (Evas_Object*) (*itBegin)->data){
						evas_object_layer_set(card,i);
						evas_object_move(card,rel_x2,rel_y);
					}
					i++;
					itBegin++;
				}
				rel_x2 += eli->card_w + spacing;
				number++;
				break;
				
			case SEPERATOR:
				
				rel_x2 += eli->card_w + spacing;
				break;
			case ENDL:
				
				rel_y += eli->card_h + (Evas_Coord)(eli->spacing * eli->card_w);
				line++;
				spacing = _elitaire_spacing_abs_calc(eli,line);
				rel_x2 = rel_x;
				
				break;
			case ALL:
				evas_object_move(eli->stacks[number],rel_x2,rel_y);
				
				itBegin = eli->game->getStackBegin(number);
				itEnd = eli->game->getStackEnd(number);
				rel_y2 = rel_y;
				while(itBegin != itEnd){
					if(card = (Evas_Object*) ((*itBegin)->data)){
						evas_object_layer_set(card,i);
						evas_object_move(card,rel_x2,rel_y2);
						
						rel_y2 += offset;
						remember_card = card;
					}
		
					itBegin++;
					i++;
				}
				rel_x2 += eli->card_w + spacing;
				remember_card = NULL;
				number++;
				i=1;
				break;
		}
		iord++;
	}
}

void _elitaire_stacks_reinit(Elitaire* eli)
{
	vector<stackLook>::const_iterator iord = eli->game->getStackLooksBegin();
	vector<stackLook>::const_iterator iend = eli->game->getStackLooksEnd();
	
	int number = 0, line = 1, i = 1;
	Evas_Coord rel_x = eli->x + (Evas_Coord) (eli->card_w * eli->margin_h);
	Evas_Coord rel_y = eli->y + (Evas_Coord) (eli->card_h * eli->margin_v);
	Evas_Coord rel_x2, offset;
	Evas_Coord spacing;
	
	offset = _elitaire_card_offset_get(eli);
	
	rel_x2 = rel_x;
	spacing = _elitaire_spacing_abs_calc(eli,line);
	
	
	while(iord != iend)
	{
		switch(*iord){
			case STACK:
			case LAST:
				evas_object_move(eli->stacks[number],rel_x2,rel_y);
				rel_x2 += eli->card_w + spacing;
				number++;
				break;
			case SEPERATOR:
				rel_x2 += eli->card_w + spacing;
				break;
			case ENDL:
				rel_y += eli->card_h + (Evas_Coord)(eli->spacing * eli->card_w);
				line++;
				spacing = _elitaire_spacing_abs_calc(eli,line);
				rel_x2 = rel_x;
				
				break;
			case ALL:
				evas_object_move(eli->stacks[number],rel_x2,rel_y);
				rel_x2 += eli->card_w + spacing;
				number++;
				i=1;
				break;
		}
		iord++;
	}
}

void _elitaire_cards_del(Elitaire * eli)
{
	int i;
	Evas_Object * card;
	playingCard * pcard;
	
	for(i=0;i < eli->num_cards;i++){
		pcard = eli->game->getCard(i,0);
		if(card = (Evas_Object*) pcard->data) {
			evas_object_del(card);
		}
	}	
}

void _elitaire_stacks_del(Elitaire * eli)
{
	int i;
	
	for(i=0;i < eli->num_stacks;i++) evas_object_del(eli->stacks[i]);
	if(eli->stacks) free(eli->stacks);
}

void _elitaire_force_resize(Elitaire * eli)
{
	int  i;
	Evas_Object * card;
	playingCard * pcard;
	
	eli->card_w = (Evas_Coord) (eli->w / ((float) eli->max_cols + eli->spacing * ((float) (eli->max_cols -1.0)) + (float) (2.0 * eli->margin_h)));
	eli->card_h = (Evas_Coord) (((float) eli->card_w) / ((float) eli->card_max_w) * eli->card_max_h);

	/* resizing the cards */
	for(i=0;i < eli->num_cards;i++){
		pcard = eli->game->getCard(i,0);
		if(card = (Evas_Object*) pcard->data){
			evas_object_resize(card, eli->card_w, eli->card_h);
			
		}
	}	
	/*resizing the stacks */
	for(i=0;i < eli->num_stacks;i++){
		evas_object_resize(eli->stacks[i], eli->card_w, eli->card_h);
	}
}


/* *********************************************************************************************** */
/* Mouse Actions Callbacks								        */
/* *********************************************************************************************** */

/** cards **************************************************************************************** */

void _elitaire_mouse_down_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	// Evas_Coord x,y,mx,my;
	Evas_Object* card = NULL;
	Evas_Event_Mouse_Down *ev = (Evas_Event_Mouse_Down*) event_info;
	
	card = (Evas_Object*) data;
	
	if(card){
		if(ev->button == 1 && elitaire_card_moveable(card)){
			elitaire_card_chain_make(card);
			elitaire_card_dragged(card);
		}
		else if(ev->button == 2){
			Evas_Object* next_card = NULL;
			next_card = elitaire_card_next_card_get(card);
			if(next_card){
				elitaire_card_chain_make(next_card);
				elitaire_card_fade(next_card);
			}
		}
	}
}

void _elitaire_mouse_up_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	
	Evas_Object* card 	= NULL;
	Evas_Event_Mouse_Up *ev = (Evas_Event_Mouse_Up*) event_info;
	Elitaire * eli;
	
	card = (Evas_Object*) data;

	if(card){
		eli = elitaire_card_elitaire_get(card);
		
		switch(ev->button){
			case 1:
				eli->game->clickOn(elitaire_card_playingCard_get(card));
				break;
			case 3:
				eli->game->rightClickOn(elitaire_card_playingCard_get(card));
				break;
		}
	}
}

/** stacks *************************************************************************************** */

void _elitaire_stack_mouse_up_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{

	Evas_Coord x,y;
	int clstack;
	Evas_Event_Mouse_Up *ev = (Evas_Event_Mouse_Up*) event_info;
	Elitaire * eli = (Elitaire*) data;
	
	if(eli){
		if(ev->button == 1){
			evas_pointer_canvas_xy_get(e, &x,  &y);
			
			clstack = elitaire_clicked_stack_get(eli,x,y);
		
			if(clstack < 0 ) return;
			eli->game->clickOn(clstack);
		}
	}
}

/** callbacks for cardGame ***/

void _elitaire_move_cb(playingCard* pcard)
{
	Evas_Object * card;
	card = (Evas_Object*) pcard->data;
	
	if(card){
		Evas_Coord x,y;
		elitaire_card_xy_get(card,&x,&y);
		elitaire_card_chain_make(card);
		elitaire_card_slide(card,x,y);
	}
}
void _elitaire_flip_cb(playingCard* pcard)
{
	Evas_Object * card;
	card = (Evas_Object*) pcard->data;
	
	if(card){
		elitaire_card_flip(card);
	}
}

void _elitaire_move_no_ani_cb(playingCard* pcard)
{
	Evas_Object * card;
	card = (Evas_Object*) pcard->data;
	
	if(card){
		Elitaire * eli;
		eli = elitaire_card_elitaire_get(card);
		_elitaire_reinit(eli);
	}
}
void _elitaire_flip_no_ani_cb(playingCard* pcard)
{
	Evas_Object * card;
	card = (Evas_Object*) pcard->data;
	
	if(card){
		elitaire_card_reinit(card);
	}
}
/*** smart object handler functions ***/

Evas_Smart * _elitaire_smart_get()
{
  if (smart) return smart;

  smart = evas_smart_new ("Elitaire",
                          _elitaire_add,
                          _elitaire_del,
                          _elitaire_layer_set,
                          _elitaire_raise,
                          _elitaire_lower,
                          _elitaire_stack_above,
                          _elitaire_stack_below,
                          _elitaire_move,
                          _elitaire_resize,
                          _elitaire_show,
                          _elitaire_hide,
                          _elitaire_color_set,
                          _elitaire_clip_set,
                          _elitaire_clip_unset,
                          NULL
                          );

  return smart; 
}

void _elitaire_add(Evas_Object *o)
{
  Elitaire *data;
  data = (Elitaire*) calloc(1,sizeof(Elitaire));
  evas_object_smart_data_set(o,data);
	
}


void _elitaire_del(Evas_Object *o)
{
	Elitaire *eli;	
	
	eli = (Elitaire*) evas_object_smart_data_get(o);
	if(!eli) return;
	
	_elitaire_cards_del(eli);
	evas_object_del(eli->clip);
	_elitaire_stacks_del(eli);
	
	delete eli->game;
	
	free(eli);
}

void _elitaire_layer_set(Evas_Object *o, int l)
{
 /* Elitaire *data;
  
  data = evas_object_smart_data_get(o);

  evas_object_layer_set(NULL, l);*/
}

void _elitaire_raise(Evas_Object *o)
{
  /*Elitaire *data;
  
  data = evas_object_smart_data_get(o);

  evas_object_layer_raise(NULL);*/
}

void _elitaire_lower(Evas_Object *o)
{
  /*Elitaire *data;
  
  data = evas_object_smart_data_get(o);

  evas_object_layer_lower(NULL);*/
}

void _elitaire_stack_above(Evas_Object *o, Evas_Object *above)
{
  /*Elitaire *data;
  
  data = evas_object_smart_data_get(o);

  evas_object_layer_stack_above(NULL, above);*/
}

void _elitaire_stack_below(Evas_Object *o, Evas_Object *below)
{
  /*Elitaire *data;
  
  data = evas_object_smart_data_get(o);

  evas_object_layer_stack_below(NULL, below);*/
}

void _elitaire_move(Evas_Object *o, Evas_Coord x, Evas_Coord y)
{
	Elitaire *eli;
  
	eli = (Elitaire*) evas_object_smart_data_get(o);
	if(!eli) return;
		
	evas_object_move(eli->clip,x,y);
	
	if(eli->y == y && eli->x == x) return; 
	eli->x = x;
	eli->y = y;	
	
	if( eli->w == 0 || eli->h == 0) return;
	if(eli->deal) _elitaire_reinit(eli);
	else _elitaire_stacks_reinit(eli);
}

void _elitaire_resize(Evas_Object *o, Evas_Coord w, Evas_Coord h)
{
	Elitaire *eli;
  
	eli = (Elitaire*) evas_object_smart_data_get(o);
	
	evas_object_resize(eli->clip,w,h);
	
	if(eli->w == w) {eli->h = h; return;}
		
	eli->w = w;
	eli->h = h;
	
	_elitaire_force_resize(eli);
	
	if(eli->deal) _elitaire_reinit(eli);
	else _elitaire_stacks_reinit(eli);
}

void _elitaire_show(Evas_Object *o)
{
	Elitaire *eli;
  
	eli = (Elitaire*) evas_object_smart_data_get(o);

	int  i;
	
	evas_object_show(eli->clip);
	
	if(!eli->deal){
		Evas_Object * card;
		playingCard * pcard;
		
		for(i=0;i < eli->num_cards;i++){
		pcard = eli->game->getCard(i,0);
			if(card = (Evas_Object*) pcard->data) {
				evas_object_move(card,eli->x + eli->w/2, eli->y + eli->h +1 );
			}
		}
	}
	for(i=0;i < eli->num_stacks;i++){
		evas_object_show(eli->stacks[i]);
	}
}

void _elitaire_hide(Evas_Object *o)
{
	Elitaire *eli;
  
	eli = (Elitaire*) evas_object_smart_data_get(o);

	int  i;
		
	evas_object_hide(eli->clip);
	
	for(i=0;i < eli->num_stacks;i++){
		evas_object_hide(eli->stacks[i]);
	}
}

void _elitaire_color_set(Evas_Object *o, int r, int g, int b, int a)
{
  /*Elitaire *data;
  
  data = evas_object_smart_data_get(o);

  evas_object_color_set(NULL, r, g, b, a);*/
}

void _elitaire_clip_set(Evas_Object *o, Evas_Object *clip)
{
  Elitaire *data;
  
  data = (Elitaire*) evas_object_smart_data_get(o);

  evas_object_clip_set(data->clip, clip);
}

void _elitaire_clip_unset(Evas_Object *o)
{
  Elitaire *data;
  
  data = (Elitaire*) evas_object_smart_data_get(o);

  evas_object_clip_unset(data->clip);
}
