 
#include"Elitaire.h"
#define SQR(x) ((x)*(x))


/* ********************************************************************************************************* */
/* Declarations										          */
/* ********************************************************************************************************* */

/* smart object handlers */
void _elitaire_card_add(Evas_Object *o);
void _elitaire_card_del(Evas_Object *o);
void _elitaire_card_layer_set(Evas_Object *o, int l);
void _elitaire_card_raise(Evas_Object *o);
void _elitaire_card_lower(Evas_Object *o);
void _elitaire_card_stack_above(Evas_Object *o, Evas_Object *above);
void _elitaire_card_stack_below(Evas_Object *o, Evas_Object *below);
void _elitaire_card_move(Evas_Object *o, Evas_Coord x, Evas_Coord y);
void _elitaire_card_resize(Evas_Object *o, Evas_Coord w, Evas_Coord h);
void _elitaire_card_show(Evas_Object *o);
void _elitaire_card_hide(Evas_Object *o);
void _elitaire_card_color_set(Evas_Object *o, int r, int g, int b, int a);
void _elitaire_card_clip_set(Evas_Object *o, Evas_Object *clip);
void _elitaire_card_clip_unset(Evas_Object *o);

/* keep a global copy of this, so it only has to be created once */
static Evas_Smart *smart;

/*** smart object handler function ***/
Evas_Smart * _elitaire_card_smart_get();

/*** some internals for the job managment ***/
_Elitaire_Card_Job *	_elitaire_card_job_new();
void			_elitaire_card_job_add(Evas_Object* card, _Elitaire_Card_Job* job);
_Elitaire_Card_Job *	_elitaire_card_job_get(Evas_Object* card);
void			_elitaire_card_job_activate(Evas_Object* card);
void 			_elitaire_card_job_first_del(Evas_Object* card);

/*** other internals ***/
void		_elitaire_card_chain_del(Evas_Object* card);
int 		_elitaire_card_chain_length_calc(Evas_Object* card);
void 		_elitaire_card_chain_next_set(Evas_Object* o, Evas_Object* ncard);
Evas_Object * 	_elitaire_card_chain_next_get(Evas_Object* o);
void		_elitaire_card_shadow_on(Evas_Object* card);
void		_elitaire_card_shadow_off(Evas_Object* card);
void		_elitaire_card_drag(Evas_Object* card);
void		_elitaire_card_drop(Evas_Object* card);
void 		_elitaire_card_fade_out(Evas_Object* card);
void   		_elitaire_card_fade_wait(Evas_Object* card);
void		_elitaire_card_fade_in(Evas_Object* card);

/*** callbacks ***/
int _elitaire_card_slide_cb	(void * data);
int _elitaire_card_flip_cb	(void * data);
int _elitaire_card_dragged_cb	(void * data);
int _elitaire_card_drag_cb	(void * data);
int _elitaire_card_drop_cb	(void * data);
int _elitaire_card_wait_cb	(void * data);
int _elitaire_card_fade_in_cb	(void * data);
int _elitaire_card_fade_out_cb  (void * data);


/* ********************************************************************************************************* */
/* external API 										          */
/* ********************************************************************************************************* */

Evas_Object * elitaire_card_new(Elitaire *eli, playingCard* pcard)
{
	Evas_Object *elitaire_card;
	Elitaire_Card * data = NULL;
	char buffer[3];

	elitaire_card = evas_object_smart_add(eli->evas, _elitaire_card_smart_get());
	data = (Elitaire_Card*) evas_object_smart_data_get(elitaire_card);
	
	if(data){
		itoa( pcard->getName(), buffer, 2 );
		data->image = edje_object_add(eli->evas);
		data->pcard = pcard;
		data->next_card = NULL;
		data->eli = eli;
		data->shadow = NULL;
		if(data->pcard->isVisible()){
			if(!edje_object_file_set(data->image,eli->file,buffer)) 
				fprintf( stderr, _("Elitaire Error: Card %s is missed!\n"),buffer);
		}
		else {if(!edje_object_file_set(data->image,eli->file,"bg")) 
				fprintf( stderr, _("Elitaire Error: The bg pic is missed!\n"));}
	}
	
	return elitaire_card;
}



void elitaire_card_playingCard_set(Evas_Object* o,playingCard* pcard)
{
	Elitaire_Card *data = NULL;
	data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
	if(data){
		data->pcard = pcard;
	}
}

playingCard * elitaire_card_playingCard_get(Evas_Object* o)
{
	Elitaire_Card *data = NULL;
	data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
	if(data){
		return data->pcard;
	}
	return NULL;
}

void elitaire_card_callback_add(Evas_Object* o, Evas_Callback_Type type, 
				void(*func)(void *data, Evas *e, Evas_Object *obj, void *event_info))
{
	Elitaire_Card *data = NULL;
	data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
	if(data){
		evas_object_event_callback_add(data->image, type, func, o);
	}
}

Evas_Bool elitaire_card_moveable(Evas_Object* card)
{
	Elitaire_Card *data = NULL;
	data = (Elitaire_Card*) evas_object_smart_data_get(card);
	
	if(data){
		return data->eli->game->moveableCard(data->pcard);
	}
	return 0;
}


Elitaire * elitaire_card_elitaire_get(Evas_Object* card)
{
	Elitaire_Card *data = NULL;
	data = (Elitaire_Card*) evas_object_smart_data_get(card);
	
	if(data) return data->eli;
	else return NULL;
}

void elitaire_card_chain_make(Evas_Object* card)
{
	Elitaire_Card *data = NULL;
	vector<playingCard*>::const_iterator it, it_end;
	Evas_Object* next_card = NULL, * owner_card;
	
	data = (Elitaire_Card*) evas_object_smart_data_get(card);
	
	if(data){
		it = data->pcard->myStack->begin();
		it_end = data->pcard->myStack->end();	
		
		/* Go to the card we are talking bout */
		while(*it != data->pcard){
			it++;
		}
		
		owner_card = card;
		it++;
		while(it != it_end){
			next_card = (Evas_Object*) (*it)->data;
			_elitaire_card_chain_next_set(owner_card,next_card);
			owner_card = next_card;
			it++;
		}
	}
}

void elitaire_card_chain_del(Evas_Object* card)
{
	_Elitaire_Card_Job * job;
	
	job = _elitaire_card_job_new();
	job->type = DEL_CHAIN;
	_elitaire_card_job_add(card,job);
	
}

/*void elitaire_card_reinit(Evas_Object* card)
{
	Elitaire_Card *data = NULL;
	char buffer[3];
  
	if(data = (Elitaire_Card*) evas_object_smart_data_get(card)){
		if(data->pcard->isVisible()){
			itoa(data->pcard->getName(), buffer, 2 );
			if(!edje_object_file_set(data->image,data->eli->file,buffer)) std::cerr << "Card " << buffer << " missed" << std::endl;
		}
		else {if(!edje_object_file_set(data->image,data->eli->file,"bg")) std::cerr << "bg pic missed" << std::endl;}
	}
}*/

void elitaire_card_reinit(Evas_Object* card)
{
	Elitaire_Card *data = NULL;
	char buffer[3];
	
	data = (Elitaire_Card*) evas_object_smart_data_get(card);
  
	if(data){
		Evas_Coord x,y,w,h;
		Evas_Object* new_image;
		int l;
		evas_object_geometry_get(data->image,&x,&y,&w,&h);
		l = evas_object_layer_get(data->image);
		new_image = edje_object_add(data->eli->evas);
				
		if(data->pcard->isVisible()){
			itoa(data->pcard->getName(), buffer, 2 );
			if(!edje_object_file_set(new_image,data->eli->file,buffer)) 
				fprintf( stderr, _("Elitaire Error: Card %s is missed!\n"),buffer);;
		}
		else {if(!edje_object_file_set(new_image,data->eli->file,"bg")) 
			fprintf( stderr, _("Elitaire Error: The bg pic is missed!\n"));}
		
		evas_object_del(data->image);
		data->image = new_image;
		evas_object_resize(data->image,w,h);
		evas_object_move(data->image,x,y);
		evas_object_clip_set(data->image,data->eli->clip);
		evas_object_layer_set(data->image,l);
		evas_object_show(data->image);
		elitaire_card_callback_add(card, EVAS_CALLBACK_MOUSE_DOWN, _elitaire_mouse_down_cb);
		elitaire_card_callback_add(card, EVAS_CALLBACK_MOUSE_UP, _elitaire_mouse_up_cb);
	}
}

void elitaire_card_slide(Evas_Object* card, Evas_Coord x, Evas_Coord y)
{
	_Elitaire_Card_Job * job;
	Elitaire * eli;
	
	eli = elitaire_card_elitaire_get(card);
	
	job = _elitaire_card_job_new();
	job->type = SLIDE;
	job->x = x;
	job->y = y;

	_elitaire_card_drag(card);
	_elitaire_card_job_add(card,job);
	if(!eli->shadow) _elitaire_card_drop(card);
	
}

void elitaire_card_flip(Evas_Object* card)
{
	Elitaire * eli;
	
	eli = elitaire_card_elitaire_get(card);
	_Elitaire_Card_Job * job;
	job = _elitaire_card_job_new();
	job->type = FLIP;
	job->x = -1;
	_elitaire_card_drag(card);
	_elitaire_card_job_add(card,job);
	
	if(!eli->shadow) _elitaire_card_drop(card);
}

void elitaire_card_dragged(Evas_Object* card)
{
	_Elitaire_Card_Job * job;
	Evas * e;
	Evas_Coord mx,my,x,y;
	Elitaire * eli;
	
	eli = elitaire_card_elitaire_get(card);	
	e = evas_object_evas_get(card);
	
	evas_object_geometry_get(card, &x, &y,NULL,NULL);
	evas_pointer_canvas_xy_get (e, &mx,  &my);
	
	job = _elitaire_card_job_new();
	job->type = DRAGGED;
	job->x = mx - x;
	job->y = my - y;
	
	_elitaire_card_drag(card);
	_elitaire_card_job_add(card,job);
	if(!eli->shadow) _elitaire_card_drop(card);
	
}

void elitaire_card_fade(Evas_Object * card)
{
	_elitaire_card_fade_out(card);
	_elitaire_card_fade_wait(card);
	_elitaire_card_fade_in(card);
	elitaire_card_chain_del(card);
}

void elitaire_card_xy_get(Evas_Object * card, Evas_Coord * x, Evas_Coord * y)
{
	vector<playingCard*>::const_iterator itBegin,itEnd;
	Elitaire * eli = elitaire_card_elitaire_get(card);
	vector<stackLook>::const_iterator iord = eli->game->getStackLooksBegin();
	vector<stackLook>::const_iterator iend = eli->game->getStackLooksEnd();
	
	playingCard * pcard;
	
	int number = 0, line = 1;
	Evas_Coord rel_x = eli->x + (Evas_Coord) (eli->card_w * eli->margin_h);
	Evas_Coord rel_y = eli->y + (Evas_Coord) (eli->card_h * eli->margin_v);
	Evas_Coord rel_y2, rel_x2, offset;
	Evas_Coord spacing;
	
	offset = _elitaire_card_offset_get(eli);
	
	rel_x2 = rel_x;
	spacing = _elitaire_spacing_abs_calc(eli,line);
	
	pcard = elitaire_card_playingCard_get(card);
	
	while(iord != iend)
	{
		switch(*iord){
			case STACK:
			case LAST:
				itBegin = eli->game->getStackBegin(number);
				itEnd = eli->game->getStackEnd(number);
				
				while(itBegin != itEnd){
					if(pcard == *itBegin){
						*x = rel_x2;
						*y = rel_y;
						return;
					}
		
					itBegin++;
				}
				rel_x2 += eli->card_w + spacing;
				number++;
				break;
				
			case SEPERATOR:
				
				rel_x2 += eli->card_w + spacing;
				break;
			case ENDL:
				rel_y += eli->card_h + (Evas_Coord)(eli->spacing * eli->card_w);
				line++;
				spacing = _elitaire_spacing_abs_calc(eli,line);
				rel_x2 = rel_x;
				
				break;
			case ALL:
				
				itBegin = eli->game->getStackBegin(number);
				itEnd = eli->game->getStackEnd(number);
				rel_y2 = rel_y;
				while(itBegin != itEnd){
					if(pcard == *itBegin){
						*x = rel_x2;
						*y = rel_y2;
						return;
					}
					rel_y2 += offset;
					itBegin++;
				
				}
				rel_x2 += eli->card_w + spacing;
				number++;
				break;
		}
		iord++;
	}
}

Evas_Object * elitaire_card_next_card_get(Evas_Object* card)
{
	playingCard* pcard;
	vector<playingCard*>::const_iterator itBegin,itEnd;
	
	pcard = elitaire_card_playingCard_get(card);
	itBegin = pcard->myStack->begin();
	itEnd = pcard->myStack->end();
	
	while(*itBegin != pcard) itBegin++;
	itBegin++;
	
	if(itBegin == itEnd) return NULL;
	else return (Evas_Object*) (*itBegin)->data;
	
}
	
/*** smart object handler functions ***/

Evas_Smart * _elitaire_card_smart_get()
{
  if (smart) return smart;

  smart = evas_smart_new ("Elitaire_Card",
                          _elitaire_card_add,
                          _elitaire_card_del,
                          _elitaire_card_layer_set,
                          _elitaire_card_raise,
                          _elitaire_card_lower,
                          _elitaire_card_stack_above,
                          _elitaire_card_stack_below,
                          _elitaire_card_move,
                          _elitaire_card_resize,
                          _elitaire_card_show,
                          _elitaire_card_hide,
                          _elitaire_card_color_set,
                          _elitaire_card_clip_set,
                          _elitaire_card_clip_unset,
                          NULL
                          );

  return smart; 
}

void
_elitaire_card_add(Evas_Object *o)
{
  Elitaire_Card *data;
  
  data = (Elitaire_Card*) calloc(1,sizeof(Elitaire_Card));
  if(data){  
	  data->image = NULL;
	  data->next_card = NULL;
	  data->pcard = NULL;
	  data->eli = NULL;
	  data->shadow = NULL;
	
	  evas_object_smart_data_set(o,data);
  }
}


void
_elitaire_card_del(Evas_Object *o)
{
  Elitaire_Card *data = NULL;
  data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
  if(data){
	evas_object_del(data->image);
	if(data->shadow) evas_object_del(data->shadow);
	free(data);
  }
}

void
_elitaire_card_layer_set(Evas_Object *o, int l)
{
  Elitaire_Card *data = NULL;
  data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
  if(data){
	if(data->shadow) evas_object_layer_set(data->shadow, l++); 
	evas_object_layer_set(data->image, l);
	if(data->next_card) evas_object_layer_set(data->next_card, l+1);
  }
}

void
_elitaire_card_raise(Evas_Object *o)
{
  Elitaire_Card *data = NULL;
  data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
  if(data){
	if(data->shadow) evas_object_raise(data->shadow);
	evas_object_raise(data->image);
	if(data->next_card) evas_object_raise(data->next_card);
  }
}

void
_elitaire_card_lower(Evas_Object *o)
{
  Elitaire_Card *data = NULL;
  data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
  if(data){
	if(data->shadow) evas_object_lower(data->shadow);
	evas_object_lower(data->image);
	if(data->next_card) evas_object_lower(data->next_card);
  }
}

void
_elitaire_card_stack_above(Evas_Object *o, Evas_Object *above)
{
  Elitaire_Card *data = NULL;
  data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
  if(data){
	evas_object_stack_above(data->image, above);
	if(data->next_card) evas_object_stack_above(data->next_card, above);
  }
}

void
_elitaire_card_stack_below(Evas_Object *o, Evas_Object *below)
{
  Elitaire_Card *data = NULL;
  data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
  if(data){
	evas_object_stack_below(data->image, below);
	if(data->next_card) evas_object_stack_below(data->next_card, below);
  }
}

void
_elitaire_card_move(Evas_Object *o, Evas_Coord x, Evas_Coord y)
{
  Elitaire_Card *data = NULL;
  data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
  if(data){
	if(data->shadow){
		Evas_Coord w;
		evas_object_move(data->shadow, x, y);
		evas_object_geometry_get(data->shadow,NULL,NULL,&w,NULL);
		x -= data->eli->card_w/10;
		y -= data->eli->card_w/10;
	}
	evas_object_move(data->image, x, y);
	if(data->next_card){ 
		Evas_Coord dy;
		dy = _elitaire_card_offset_get(data->eli);
		evas_object_move(data->next_card, x, y+dy);}
  }
}


void
_elitaire_card_resize(Evas_Object *o, Evas_Coord w, Evas_Coord h)
{
  Elitaire_Card *data = NULL;
  data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
  if(data){
	if(data->shadow){
		Evas_Coord x,y;
		evas_object_resize(data->shadow, w, h);
		evas_object_geometry_get(data->shadow,&x,&y,NULL,NULL);
		x -= data->eli->card_w/10;
		y -= data->eli->card_w/10;
		evas_object_move(data->image,x,y);
	}
	
	evas_object_resize(data->image, w, h);
	
	if(data->next_card){
		Evas_Coord x,y;
		Evas_Coord dy;
		evas_object_resize(data->next_card, w, h);
		evas_object_geometry_get(data->image,&x,&y,NULL,NULL);
		dy = _elitaire_card_offset_calc(data->eli,h);
		evas_object_move(data->next_card,x,y+dy);
	}
  }
}

void
_elitaire_card_show(Evas_Object *o)
{
  Elitaire_Card *data = NULL;
  data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
  if(data){
	evas_object_show(data->image);
	if(data->next_card) evas_object_show(data->next_card);
  }
}

void
_elitaire_card_hide(Evas_Object *o)
{
  Elitaire_Card *data = NULL;
  data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
  if(data){
	evas_object_hide(data->image);
	if(data->next_card) evas_object_hide(data->next_card);
	if(data->shadow) evas_object_hide(data->shadow);
  }
}

void
_elitaire_card_color_set(Evas_Object *o, int r, int g, int b, int a)
{
  Elitaire_Card *data = NULL;
  data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
  if(data){
	evas_object_color_set(data->image, r, g, b, a);
	if(data->next_card) evas_object_color_set(data->next_card, r, g, b, a);
  }
}

void
_elitaire_card_clip_set(Evas_Object *o, Evas_Object *clip)
{
  Elitaire_Card *data = NULL;
  data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
  if(data){
	evas_object_clip_set(data->image, clip);
	if(data->next_card) evas_object_clip_set(data->next_card, clip);
	if(data->shadow) evas_object_clip_set(data->shadow, clip);
  }
}

void
_elitaire_card_clip_unset(Evas_Object *o)
{
  Elitaire_Card *data = NULL;
  data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
  if(data){
	evas_object_clip_unset(data->image);
	if(data->next_card) evas_object_clip_unset(data->next_card);
	if(data->shadow) evas_object_clip_unset(data->shadow);
  }
}

/* ************************************************************************************************************ */
/*** some interals for the job managment 							                */
/* ************************************************************************************************************ */

/* allocate memory */
_Elitaire_Card_Job * _elitaire_card_job_new()
{
	_Elitaire_Card_Job* job;
	job = (_Elitaire_Card_Job*) calloc(1,sizeof(_Elitaire_Card_Job));
	job->next_job = NULL;
	return job;
}

/* add a job at the end of the list */
void _elitaire_card_job_add(Evas_Object* card, _Elitaire_Card_Job* new_job)
{
	Elitaire_Card *data = NULL;
	data = (Elitaire_Card*) evas_object_smart_data_get(card);
	
	if(data){
		if(!data->job){
			data->job = new_job;
		}
		else{
			_Elitaire_Card_Job* search;
			_Elitaire_Card_Job* prev;
			
			for(search = data->job; search ; search = search->next_job)
				prev = search;
			
			prev->next_job = new_job;
		}
		_elitaire_card_job_activate(card);
		
	}
}

/* return the recent job */
_Elitaire_Card_Job * _elitaire_card_job_get(Evas_Object* card)
{
	Elitaire_Card *data = NULL;
	data = (Elitaire_Card*) evas_object_smart_data_get(card);
	
	if(data){
		return data->job;
	}
	return NULL;
}


/* delete the recent job and put the next job at the first position */
void _elitaire_card_job_first_del(Evas_Object* card)
{
	Elitaire_Card *data = NULL;
	data = (Elitaire_Card*) evas_object_smart_data_get(card);
	
	if(data){
		if(data->job){
			_Elitaire_Card_Job * job = NULL;
			
			job = data->job->next_job;
			if(data->job->timer) ecore_timer_del(data->job->timer);
			data->job->timer = NULL;
			free(data->job);
			if(job) data->job = job;
			else data->job = NULL;
		}
		_elitaire_card_job_activate(card);
	}
}

void _elitaire_card_job_activate(Evas_Object* card)
{
	Elitaire_Card* data = NULL;
	data = (Elitaire_Card*) evas_object_smart_data_get(card);
	
	if(data){
		if(data->job){
			int fr = data->eli->frame_rate;
			
			switch(data->job->type){
				case SLIDE:
					if(!data->job->timer){
						data->job->timer = ecore_timer_add(1.0/fr,_elitaire_card_slide_cb,card);
					} break;
				
				case DEL_CHAIN:
					_elitaire_card_chain_del(card);
					_elitaire_card_job_first_del(card);
					break;
				
				case FLIP:
					if(!data->job->timer){
						data->job->timer = ecore_timer_add(1.0/fr,_elitaire_card_flip_cb,card);
					} break;
					
				case DRAGGED:
					if(!data->job->timer){
						data->job->timer = ecore_timer_add(1.0/fr,_elitaire_card_dragged_cb,card);
					} 
					break;
					
				case DRAG:
					evas_object_layer_set(card,52);
					if(!data->job->timer && data->eli->shadow){
						_elitaire_card_shadow_on(card);
						evas_object_layer_set(card,54);
						data->job->timer = ecore_timer_add(1.0/fr,_elitaire_card_drag_cb,card);
					} 
					else { 
						_elitaire_card_job_first_del(card);
					}
					break;
					
				case DROP:
					if(data->eli->shadow){
						if(!data->job->timer){
							data->job->timer = ecore_timer_add(1.0/fr,_elitaire_card_drop_cb,card);
						}
					} 
					else { 
						elitaire_stack_layer_reinit(data->eli,data->pcard->myStack);
						_elitaire_card_job_first_del(card);
						elitaire_card_chain_del(card);
					}
					break;
				case WAIT:
					if(!data->job->timer){
						data->job->timer = ecore_timer_add(1.0/fr,_elitaire_card_wait_cb,card);
					} 
					break;
				case FADE_IN:
					if(!data->job->timer){
						data->job->timer = ecore_timer_add(1.0/fr,_elitaire_card_fade_in_cb,card);
					} 
					break;
				case FADE_OUT:
					if(!data->job->timer){
						data->job->timer = ecore_timer_add(1.0/fr,_elitaire_card_fade_out_cb,card);
					} 
					break;
			}
		}
		else if(data->shadow){ 
			_elitaire_card_drop(card);
			elitaire_card_chain_del(card);
		}
	}	
}

/* ************************************************************************************************************ */
/*  other internals 										                */
/* ************************************************************************************************************ */
void _elitaire_card_chain_del(Evas_Object* card)
{
	Evas_Object* owner_card, * next_card;
	owner_card = card;
	next_card = _elitaire_card_chain_next_get(owner_card);
	_elitaire_card_chain_next_set(owner_card,NULL);
	while(next_card != NULL){
		owner_card = next_card;
		next_card = _elitaire_card_chain_next_get(owner_card);
		_elitaire_card_chain_next_set(owner_card,NULL);
	}
}

int _elitaire_card_chain_length_calc(Evas_Object* card)
{
	playingCard* pcard;
	vector<playingCard*>::const_iterator it, it_end;
	
	pcard = elitaire_card_playingCard_get(card);
	it = pcard->myStack->begin();
	it_end = pcard->myStack->end();
	
	while(*it != pcard) it++;
	
	return it_end - it;
}

void _elitaire_card_chain_next_set(Evas_Object* o, Evas_Object* ncard)
{
	Elitaire_Card *data = NULL;
	data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
	if(data){
		data->next_card = ncard;
	}
}

Evas_Object * _elitaire_card_chain_next_get(Evas_Object* o)
{
	Elitaire_Card *data = NULL;
	data = (Elitaire_Card*) evas_object_smart_data_get(o);
	
	if(data){
		return data->next_card;
	}
	return NULL;
}

void _elitaire_card_shadow_on(Evas_Object* card)
{
	Elitaire_Card *data = NULL;
	data = (Elitaire_Card*) evas_object_smart_data_get(card);
	
	if(data){
		Evas_Coord x,y,w,h,dy;
		int n;
		
		if(data->shadow) return;
		data->shadow = edje_object_add(data->eli->evas);
		if(!edje_object_file_set(data->shadow,data->eli->file,"shadow")) 
			fprintf( stderr, _("Elitaire Error: The shadow pic is missed!\n"));
		
		n = _elitaire_card_chain_length_calc(card) - 1;
		dy = _elitaire_card_offset_get(data->eli);
	
		evas_object_geometry_get(card,&x,&y,&w,&h);
		evas_object_move(data->shadow,x,y);
		evas_object_resize(data->shadow,w,h + n*dy);
		evas_object_clip_set(data->shadow,data->eli->clip);
		evas_object_show(data->shadow);
		//evas_object_stack_below(data->image,data->shadow);
	}
}

void _elitaire_card_shadow_off(Evas_Object* card)
{
	Elitaire_Card *data = NULL;
	data = (Elitaire_Card*) evas_object_smart_data_get(card);
	
	if(data){
		if(data->shadow) evas_object_del(data->shadow);
		data->shadow = NULL;
	}
}

void _elitaire_card_drag(Evas_Object* card)
{
	_Elitaire_Card_Job * job;
	
	job = _elitaire_card_job_new();
	job->type = DRAG;
	_elitaire_card_job_add(card,job);
	
}
void _elitaire_card_drop(Evas_Object* card)
{
	_Elitaire_Card_Job * job;
	
	job = _elitaire_card_job_new();
	job->type = DROP;
	_elitaire_card_job_add(card,job);
}

void _elitaire_card_fade_out(Evas_Object* card)
{
	_Elitaire_Card_Job * job;
	job = _elitaire_card_job_new();
	job->type = FADE_OUT;
	job->x = 256;
	_elitaire_card_job_add(card,job);
}

void _elitaire_card_fade_wait(Evas_Object* card)
{
	_Elitaire_Card_Job * job;
	job = _elitaire_card_job_new();
	job->type = WAIT;
	job->x = 2;
	_elitaire_card_job_add(card,job);
}

void _elitaire_card_fade_in(Evas_Object* card)
{
	_Elitaire_Card_Job * job;
	job = _elitaire_card_job_new();
	job->type = FADE_IN;
	job->x = 30;
	_elitaire_card_job_add(card,job);
}
/* ************************************************************************************************************ */
/* callbacks 										                */
/* ************************************************************************************************************ */
int _elitaire_card_slide_cb(void * data)
{
	Evas_Object* card = NULL;
	Elitaire_Card* eli_card = NULL;
	
	card = (Evas_Object*) data;
	if(!card) return 0;
	eli_card = (Elitaire_Card*) evas_object_smart_data_get(card);
	if(!eli_card) return 0;
	
	
	if(eli_card->job){
		if(eli_card->job->timer){
			Evas_Coord new_x, new_y , to_x, to_y;
			double m;
			Evas_Coord from_x,from_y;
			
			to_x = eli_card->job->x;
			to_y = eli_card->job->y;
			evas_object_geometry_get(card,&from_x,&from_y,NULL,NULL);
			
			m = (eli_card->eli->velocity/  eli_card->eli->frame_rate)
				/ sqrt( (double) (SQR(to_x - from_x) +  SQR(to_y - from_y)));
			
			new_x = (Evas_Coord) (m * ( to_x - from_x)) + from_x;
			new_y = (Evas_Coord) (m * ( to_y - from_y)) + from_y;
			if(ABS(new_x - from_x) >= ABS(to_x-from_x) && ABS(new_y - from_y) >= ABS(to_y - from_y)){
				/* Job now ends */
				evas_object_move(card,to_x,to_y);
				_elitaire_card_job_first_del(card);
				return 0;
			}
			evas_object_move(card,new_x,new_y);
			evas_object_layer_set(card, eli_card->eli->num_cards);
			
			return 1;
		}
	}
	return 0;	
}

int _elitaire_card_flip_cb(void * data)
{
	Evas_Object* card = NULL;
	Elitaire_Card* eli_card = NULL;
	
	
	card = (Evas_Object*) data;
	if(!card) return 0;
	eli_card = (Elitaire_Card*) evas_object_smart_data_get(card);
	if(!eli_card) return 0;
	
	
	if(eli_card->job){
		if(eli_card->job->timer){
			Evas_Coord y,h;
			Evas_Coord old_x, old_w;
			Evas_Coord new_x, new_w;
			
			evas_object_geometry_get(card,&old_x,&y,&old_w,&h);
			new_w = old_w + eli_card->job->x * (Evas_Coord) (eli_card->eli->velocity
							/ eli_card->eli->frame_rate);
			if(new_w < 0){
				eli_card->job->x = 1;
				new_w = abs(new_w);
				elitaire_card_reinit(card);
			}
			if(new_w >= eli_card->eli->card_w){
				/* Job now ends */
				new_w = eli_card->eli->card_w;
				new_x = old_x + (old_w - new_w)/2;				
				evas_object_resize(card, new_w,h);
				evas_object_move(card, new_x,y);
				_elitaire_card_job_first_del(card);
				return 0;
			}
			new_x = old_x + (old_w - new_w)/2;
			evas_object_resize(card, new_w,h);
			evas_object_move(card, new_x,y);
			
			return 1;
		}
	}
	return 0;	
}

int _elitaire_card_dragged_cb(void* data)
{
	Evas_Object * card = NULL;
	card = (Evas_Object*) data;
	
	if(card){
		_Elitaire_Card_Job * job;
		
		job = _elitaire_card_job_get(card);
			
		Evas_Coord mx,my;
		Evas *evas; 
		
		evas = evas_object_evas_get(card);
		evas_pointer_canvas_xy_get(evas, &mx,  &my);
		
		if((evas_pointer_button_down_mask_get(evas) -1) % 2){
			/* Job now ends */
			Evas_Coord to_x, to_y , x,y,w,h;
			Elitaire * eli;
			int clstack, moved = 0;
			
			eli = elitaire_card_elitaire_get(card);
			evas_object_geometry_get(card,&x,&y,&w,&h);
			x += w/2;
			y += h/2;
			clstack = elitaire_clicked_stack_get(eli,x,y);
		
			if(clstack >= 0)
				moved = eli->game->move(elitaire_card_playingCard_get(card), clstack);
			if(!moved){	
				if(eli->animate){
					elitaire_card_xy_get(card,&to_x,&to_y);	
					elitaire_card_slide(card,to_x,to_y);
				}
				else _elitaire_reinit(eli);
			}
			_elitaire_card_job_first_del(card);
			return 0; 		
		}
		
		evas_object_move(card, mx - job->x, my - job->y);
			
	return 1;
	}
	else return 0;
}

int _elitaire_card_drag_cb(void * data)
{
	Evas_Object * card = NULL;
	card = (Evas_Object*) data;
	
	if(card){
		Elitaire_Card * eli_card;
		if(eli_card = (Elitaire_Card*) evas_object_smart_data_get(card)){
			Evas_Coord x,y,w,d;
			Evas_Coord image_x,image_y;
			
			evas_object_geometry_get(eli_card->shadow,&x,&y,&w,NULL);
			evas_object_geometry_get(eli_card->image,&image_x,&image_y,NULL,NULL);
			
			d = (Evas_Coord) eli_card->eli->velocity / eli_card->eli->frame_rate;
			image_x -= d;
			image_y -= d;
			
			if(abs(x - image_x) >= w/10){
				/* Job now ends */
				evas_object_move(eli_card->image,x - w/10,y - w/10);
				_elitaire_card_job_first_del(card);
				return 0;
			}
			
			evas_object_move(eli_card->image,image_x,image_y);
			
			if(eli_card->next_card){ 
				Evas_Coord dy;
				dy = _elitaire_card_offset_get(eli_card->eli);
				evas_object_move(eli_card->next_card, image_x, image_y+dy);
			}
			
			return 1;
		}
	}
	return 0;
}

int _elitaire_card_drop_cb(void * data)
{
	Evas_Object * card = NULL;
	card = (Evas_Object*) data;
	
	if(card){
		Elitaire_Card * eli_card;
		if(eli_card = (Elitaire_Card*) evas_object_smart_data_get(card)){
			Evas_Coord x,y,w,d;
			Evas_Coord image_x,image_y;
			
			evas_object_geometry_get(eli_card->shadow,&x,&y,&w,NULL);
			evas_object_geometry_get(eli_card->image,&image_x,&image_y,NULL,NULL);
			
			d = (Evas_Coord) eli_card->eli->velocity / eli_card->eli->frame_rate;
			image_x += d;
			image_y += d;
			
			if(x - image_x <= 0){
				/* Job now ends */
				evas_object_move(eli_card->image,x,y);
				if(eli_card->next_card){ 
					Evas_Coord dy;
					dy = _elitaire_card_offset_get(eli_card->eli);
					evas_object_move(eli_card->next_card, x, y+dy);
				}
				_elitaire_card_shadow_off(card);
				_elitaire_card_job_first_del(card);
				elitaire_stack_layer_reinit(eli_card->eli,eli_card->pcard->myStack);
				return 0;
			}
			
			evas_object_move(eli_card->image,image_x,image_y);
			
			if(eli_card->next_card){ 
				Evas_Coord dy;
				dy = _elitaire_card_offset_get(eli_card->eli);
				evas_object_move(eli_card->next_card, image_x, image_y+dy);
			}
			return 1;
		}
	}
	return 0;
}

int _elitaire_card_wait_cb(void * data)
{
	Evas_Object * card = NULL;
	card = (Evas_Object*) data;
	
	if(card){
		_Elitaire_Card_Job * job;
		job = _elitaire_card_job_get(card);
		Evas* evas;
		
		evas = evas_object_evas_get(card);
		
		if(evas_pointer_button_down_mask_get(evas) == job->x) return 1; 		
		else {
			_elitaire_card_job_first_del(card);
			return 0;
		}
	}
	return 0;
}

int _elitaire_card_fade_in_cb(void * data)
{
	Evas_Object * card = NULL;
	card = (Evas_Object*) data;
	
	if(card){
		_Elitaire_Card_Job * job;
		Elitaire * eli;
		
		job = _elitaire_card_job_get(card);
		eli = elitaire_card_elitaire_get(card);
		
		job->x += (int) (eli->velocity / eli->frame_rate);
		
		if( job->x >= 256) {
			evas_object_color_set(card, 256, 256, 256, 256);
			_elitaire_card_job_first_del(card);
			return 0;
		}
		else {
			evas_object_color_set(card, 256, 256, 256, job->x);
			return 1;
		}
	}
	else return 0;
}
int _elitaire_card_fade_out_cb(void * data)
{
	Evas_Object * card = NULL;
	card = (Evas_Object*) data;
	
	if(card){
		_Elitaire_Card_Job * job;
		Elitaire * eli;
		
		job = _elitaire_card_job_get(card);
		eli = elitaire_card_elitaire_get(card);
		
		job->x -= (int) (eli->velocity / eli->frame_rate);
		
		if( job->x <= 30) {
			evas_object_color_set(card, 256, 256, 256, 30);
			_elitaire_card_job_first_del(card);
			return 0;
		}
		else {
			evas_object_color_set(card, 256, 256, 256, job->x);
			return 1;
		}
	}
	else return 0;
}
