#include<Ewl.h>
#include"ewl_context_menu.h"

void ewl_context_menu_expand_cb(Ewl_Widget *w, void *ev_data ,
                                        void *user_data );
void ewl_context_menu_configure_cb(Ewl_Widget * w, void *ev_data,
                                        void *user_data);
void ewl_context_menu_destroy_cb(Ewl_Widget * w, void *ev_data,
					void *user_data);
					
Ewl_Widget* ewl_context_menu_new()
{
	Ewl_Context_Menu * cmenu;

        cmenu = EWL_CONTEXT_MENU(calloc(sizeof(Ewl_Context_Menu),1));
        if (!cmenu) return NULL;
	
        ewl_context_menu_init(cmenu);
	
	return EWL_WIDGET(cmenu);
}

void ewl_context_menu_init(Ewl_Context_Menu* cmenu)
{
		
	ewl_window_init(EWL_WINDOW(cmenu));
	
	ewl_widget_inherit(EWL_WIDGET(cmenu), "context_menu");
	
	ewl_window_borderless_set(EWL_WINDOW(cmenu));
	ewl_widget_layer_set(EWL_WIDGET(cmenu), 1000);
	ewl_widget_internal_set(EWL_WIDGET(cmenu), TRUE);
        ewl_widget_appearance_set(EWL_WIDGET(cmenu), "menu");
	
	cmenu->x = 0;
	cmenu->y = 0;
	
	/* the pop box */
	cmenu->popbox = ewl_vbox_new();
	
	ewl_box_orientation_set(EWL_BOX(cmenu->popbox),
                                EWL_ORIENTATION_VERTICAL);
        ewl_object_fill_policy_set(EWL_OBJECT(cmenu->popbox),
                                   EWL_FLAG_FILL_NONE);
        ewl_object_alignment_set(EWL_OBJECT(cmenu->popbox),
                                 EWL_FLAG_ALIGN_LEFT | EWL_FLAG_ALIGN_TOP);
	ewl_widget_show(cmenu->popbox);
	ewl_container_child_append(EWL_CONTAINER(cmenu),
                                           cmenu->popbox);
	ewl_container_redirect_set(EWL_CONTAINER(cmenu),
				   EWL_CONTAINER(cmenu->popbox));
	
	/* add callbacks */
        ewl_callback_append(EWL_WIDGET(cmenu), EWL_CALLBACK_REALIZE,
                            ewl_context_menu_expand_cb, NULL);

        ewl_callback_append(EWL_WIDGET(cmenu), EWL_CALLBACK_DESTROY,
                           ewl_context_menu_destroy_cb, NULL);
	ewl_callback_append(EWL_WIDGET(cmenu), EWL_CALLBACK_CONFIGURE,
                           ewl_context_menu_configure_cb, NULL);
		

}

void ewl_context_menu_move(Ewl_Context_Menu * cmenu, int x, int y)
{
	cmenu->x = x;
	cmenu->y = y;
	
	if(REALIZED(cmenu)) ewl_window_move(EWL_WINDOW(cmenu),x,y);
}

void ewl_context_menu_expand_cb(Ewl_Widget *w, void *ev_data,
                                        void *user_data)
{
	Ewl_Widget *child;
        Ewl_Container *pb;
        Ewl_Context_Menu *menu = EWL_CONTEXT_MENU(w);
        Ewl_Menu_Item *item = NULL;

        
        pb = EWL_CONTAINER(menu->popbox);
        
        ecore_list_goto_first(pb->children);
         
        /*
         * Give all the items in the submenu a reference to this popup.
         */
        while ((child = ecore_list_next(pb->children))) {
                if (ewl_widget_type_is(child, "menuitem")) {
                        item = EWL_MENU_ITEM(child);
                        item->inmenu = EWL_WIDGET(menu);
                }
        }
	
        ewl_widget_focus_send(menu->popbox);
}



void ewl_context_menu_configure_cb(Ewl_Widget * w, void *ev_data,
                                        void *user_data)
{
	int x,y;
	
	x = EWL_CONTEXT_MENU(w)->x;
	y = EWL_CONTEXT_MENU(w)->y;
	
	ewl_window_move(EWL_WINDOW(w),x,y);
	
	ewl_window_raise(EWL_WINDOW(w));
}

void ewl_context_menu_destroy_cb(Ewl_Widget * w, void *ev_data,
					void *user_data)
{
	Ewl_Context_Menu * menu;
	
	menu = EWL_CONTEXT_MENU(w);
	
	if(menu->popbox)
		ewl_widget_destroy(menu->popbox);
	
}
