#include"Esmart_Resize.h"
#include <Esmart/Esmart_Draggies.h>
#include"Eli_App.h"

/* internal functions */
void _eli_app_gui_swallow(Evas_Object* gui,const char * part,Evas_Object* o);
/* callback  declaration */
void _new_game_cb(void *data, Evas_Object *o, const char *emission, const char *source);
void _pipe_cb(void* data, Evas_Object* o, const char *emission, const char *source);
void _quit_cb(void* data, Evas_Object* o, const char *emission, const char *source);
void _velocity_drag_stop_cb(void *data, Evas_Object *o, const char *emission, const char *source);
void _velocity_drag_cb(void *data, Evas_Object *o, const char *emission, const char *source);
void _undo_cb(void* data, Evas_Object* o, const char *emission, const char *source);
void _mouse_up_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
void _mouse_down_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);

void eli_app_gui_make(Eli_App* eap)
{
	Evas_Coord w,h;
	Evas_Object *o;
	char *gui_theme;
	int is_shaped = 0;

	eap->gui  = edje_object_add(eap->evas);
	
	gui_theme  = ecore_config_theme_with_path_from_name_get(eap->theme.gui.current);
	
	if(edje_object_file_set(eap->gui,gui_theme,"main")) { 
		evas_object_move(eap->gui,0,0);
		edje_object_size_min_get(eap->gui,&w,&h);
		evas_object_resize(eap->gui,w,h);
		evas_object_layer_set(eap->gui,-4);
		edje_object_signal_callback_add(eap->gui,"quit","*",_quit_cb,eap);
		edje_object_signal_callback_add(eap->gui,"undo","*",_undo_cb,eap);
		evas_object_show(eap->gui);
	}
	else {
		fprintf( stderr, _("Elitaire Error: unable to find %s theme!\n") ,gui_theme);
		return;
	}
	
	/* some ecore_evas stuff*/
	const char * shaped;
	const char * borderless;
	
	shaped = edje_object_data_get(eap->gui,"shaped");
	borderless = edje_object_data_get(eap->gui,"borderless");
	
	if(!shaped) ecore_evas_shaped_set(eap->ee, 0);
	else if(!strcmp("1",shaped)){ 
		ecore_evas_shaped_set(eap->ee, 1);
		is_shaped = 1;
	}
	else ecore_evas_shaped_set(eap->ee, 0);
		
	if(!borderless) ecore_evas_borderless_set(eap->ee, 0);
	else if(!strcmp("1", borderless))
		ecore_evas_borderless_set(eap->ee, 1);
	else ecore_evas_borderless_set(eap->ee, 0);
		
	ecore_evas_resize(eap->ee,w,h);
	ecore_evas_size_min_set(eap->ee, w, h);
	
	/*the menu bar*/
	if(edje_object_part_exists(eap->gui,"elitaire_menu_bar") ){
		o = edje_object_add(eap->evas);
		if(edje_object_file_set(o,gui_theme,"menubar")){
			Evas_Object* container;
			
			container = esmart_container_new(ecore_evas_get(eap->ee));
			esmart_container_direction_set(container, CONTAINER_DIRECTION_VERTICAL);
			esmart_container_alignment_set(container, CONTAINER_ALIGN_TOP);
			evas_object_show(container);
			
			_eli_app_gui_swallow(eap->gui,"elitaire_menu_bar", o);
			_eli_app_gui_swallow(o,"elitaire_new_box", container);
			
			eli_app_menu_container_fill(eap,container);
			
			edje_object_signal_callback_add(o,"*","*",_pipe_cb,eap);
			edje_object_signal_callback_add(o,"drag,stop","elitaire_velocity_slider",_velocity_drag_stop_cb,eap);
			edje_object_signal_callback_add(o,"drag","elitaire_velocity_slider",_velocity_drag_cb,eap);
			_velocity_drag_cb(eap,o,"drag","elitaire_velocity_slider");
			
			if(!is_shaped) evas_object_layer_set(o,152);
		}
	}
	
	/* the theme bar */
	if(edje_object_part_exists(eap->gui,"elitaire_theme_bar") ){
		o = edje_object_add(eap->evas);
		if(edje_object_file_set(o,gui_theme,"themebar")){
			Evas_Object* container;
			
			container = esmart_container_new(ecore_evas_get(eap->ee));
			esmart_container_direction_set(container, CONTAINER_DIRECTION_VERTICAL);
			esmart_container_alignment_set(container, CONTAINER_ALIGN_TOP);
			evas_object_show(container);
			
			
			_eli_app_gui_swallow(eap->gui,"elitaire_theme_bar", o);
			_eli_app_gui_swallow(o,"elitaire_theme_box", container);
			
			eli_app_theme_container_fill(eap,container);
						
			edje_object_signal_callback_add(o,"*","*",_pipe_cb,eap);
			
			if(!is_shaped) evas_object_layer_set(o,151);
		}
	}
	/* the card bar */
	if(edje_object_part_exists(eap->gui,"elitaire_card_bar") ){
		o = edje_object_add(eap->evas);
		if(edje_object_file_set(o,gui_theme,"cardbar")){
			Evas_Object* container;
			
			container = esmart_container_new(ecore_evas_get(eap->ee));
			esmart_container_direction_set(container, CONTAINER_DIRECTION_VERTICAL);
			esmart_container_alignment_set(container, CONTAINER_ALIGN_TOP);
			evas_object_show(container);
			
			_eli_app_gui_swallow(eap->gui,"elitaire_card_bar", o);
			_eli_app_gui_swallow(o,"elitaire_cards_box", container);
			
			eli_app_cards_container_fill(eap,container);
						
			edje_object_signal_callback_add(o,"*","*",_pipe_cb,eap);
			
			if(!is_shaped) evas_object_layer_set(o,150);
		}
	}
	/* theme parts for moving the window */
	if(edje_object_part_exists(eap->gui,"elitaire_drag")){
		o = esmart_draggies_new(eap->ee);
		esmart_draggies_button_set(o,1);
		_eli_app_gui_swallow(eap->gui,"elitaire_drag" ,o);
	}
	
	char* resize_types[] = {
                "elitaire_resize_left",
                "elitaire_resize_right",
                "elitaire_resize_top",
                "elitaire_resize_bottom",
                "elitaire_resize_left_top",
                "elitaire_resize_right_top",
                "elitaire_resize_left_bottom",
                "elitaire_resize_right_bottom"};
                
        Esmart_Resize_Type rt[] = {
                ESMART_RESIZE_LEFT,
                ESMART_RESIZE_RIGHT,
                ESMART_RESIZE_TOP,
                ESMART_RESIZE_BOTTOM,
                ESMART_RESIZE_LEFT_TOP,
                ESMART_RESIZE_RIGHT_TOP,
                ESMART_RESIZE_LEFT_BOTTOM,
                ESMART_RESIZE_RIGHT_BOTTOM};

	/* theme parts for resizing the window*/
	for(int i = 0; i<8 ; i++){
		if(edje_object_part_exists(eap->gui,resize_types[i])){
			o = esmart_resize_new(eap->ee);
			esmart_resize_button_set(o,1);
			esmart_resize_type_set(o,rt[i]);
			_eli_app_gui_swallow(eap->gui,resize_types[i], o);
		}
	}
	
	/* theme parts for the context menu */
	if(edje_object_part_exists(eap->gui,"elitaire_context_menu")){
		o = evas_object_rectangle_add(eap->evas);
		evas_object_color_set(o,0,0,0,0);
		evas_object_repeat_events_set (o, 1);
		evas_object_event_callback_add(o,EVAS_CALLBACK_MOUSE_DOWN,_mouse_down_cb,eap);
		evas_object_event_callback_add(o,EVAS_CALLBACK_MOUSE_UP,_mouse_up_cb,eap);
		_eli_app_gui_swallow(eap->gui,"elitaire_context_menu" ,o);
		evas_object_layer_set(o,-1);
	}
	
	int vel;
	vel = ecore_config_int_get("velocity");
	eli_app_velocity_set(eap,vel);
	
	if(eap->elitaire) _eli_app_gui_swallow(eap->gui,"card_field", eap->elitaire);
}

void eli_app_gui_del(Eli_App* eap)
{
	Evas_Object * o;
	
	char* unsw[] = {
		"elitaire_context_menu",
		"elitaire_drag",
                "elitaire_resize_left",
                "elitaire_resize_right",
                "elitaire_resize_top",
                "elitaire_resize_bottom",
                "elitaire_resize_left_top",
                "elitaire_resize_right_top",
                "elitaire_resize_left_bottom",
                "elitaire_resize_right_bottom"};
		
	for(int i = 0; i<10; i++){
		o = edje_object_part_swallow_get(eap->gui,unsw[i]);
		if(o){ 
			edje_object_part_unswallow(eap->gui,o);
			evas_object_del(o);
		}
	}
	
	/* containers need a special treatment */
	o = edje_object_part_swallow_get(eap->gui,"elitaire_card_bar");
	if(o){
		Evas_Object* container;
		container = edje_object_part_swallow_get(o,"elitaire_cards_box");
		edje_object_part_unswallow(o,container);
		edje_object_part_unswallow(eap->gui,o);
		evas_object_del(container);
		evas_object_del(o);
	}
	o = edje_object_part_swallow_get(eap->gui,"elitaire_theme_bar");
	if(o){
		Evas_Object* container;
		container = edje_object_part_swallow_get(o,"elitaire_theme_box");
		edje_object_part_unswallow(o,container);
		edje_object_part_unswallow(eap->gui,o);
		evas_object_del(container);
		evas_object_del(o);
	}
	
	o = edje_object_part_swallow_get(eap->gui,"elitaire_menu_bar");
	if(o){
		Evas_Object* container;
		container = edje_object_part_swallow_get(o,"elitaire_new_box");
		edje_object_part_unswallow(o,container);
		edje_object_part_unswallow(eap->gui,o);
		evas_object_del(container);
		evas_object_del(o);
	}
	
	if(edje_object_part_swallow_get(eap->gui,"card_field"))
		edje_object_part_unswallow(eap->gui,eap->elitaire);
	
	evas_object_del(eap->gui);
	eap->gui = NULL;
}

void eli_app_gui_resize(Eli_App* eap, int w,int h)
{
	evas_object_resize(eap->gui,(Evas_Coord) w,(Evas_Coord) h);
}

/* internal */
void _eli_app_gui_swallow(Evas_Object * gui, const char * part, Evas_Object* o)
{
	Evas_Coord x,y,w,h;
	
	edje_object_part_geometry_get(gui, part , &x,&y,&w,&h);
	evas_object_move(o,x,y);
	evas_object_resize(o, w, h);
	evas_object_show(o);
	edje_object_part_swallow(gui,part,o);
}

/* callbacks */
void _pipe_cb(void* data, Evas_Object* o, const char *emission, const char *source)
{
	Eli_App* eap;
	eap = (Eli_App*) data;
	edje_object_signal_emit(eap->gui,emission,source);
}

void _quit_cb(void* data, Evas_Object* o, const char *emission, const char *source)
{
	ecore_main_loop_quit();
}

void _velocity_drag_stop_cb(void *data, Evas_Object *o, const char *emission, const char *source)
{
	Eli_App* eap;
	eap = (Eli_App*) data;
	
	ecore_config_int_set("velocity", eli_app_velocity_get(eap));
}

void _velocity_drag_cb(void *data, Evas_Object *o, const char *emission, const char *source)
{
	Eli_App* eap;
	eap = (Eli_App*) data;
	
	if(edje_object_part_exists(o,"elitaire_velocity_value") && eap){
		char buffer[5];
		
		snprintf(buffer,5,"%i", eli_app_velocity_get(eap));
		edje_object_part_text_set(o,"elitaire_velocity_value",buffer);
	}
}

void _undo_cb(void* data, Evas_Object* o, const char *emission, const char *source)
{
	Eli_App* eap;
	eap = (Eli_App*) data;
	
	if(!eap) return;
	if(eap->elitaire) elitaire_undo(eap->elitaire);
}

void _mouse_up_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Eli_App * eap;
	
	if(data) eap = (Eli_App*) data;
	
}
void _mouse_down_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Eli_App * eap;
	Evas_Event_Mouse_Down * ev;
	int x,y;
	
	if(data) eap = (Eli_App*) data;
	else return;
		
	ev = (Evas_Event_Mouse_Down*) event_info;
	
	switch(ev->button){
		case 3: 
			ecore_evas_geometry_get(eap->ee, &x, &y, NULL, NULL);
			x += evas_coord_world_x_to_screen(e,ev->canvas.x);
			y += evas_coord_world_y_to_screen(e,ev->canvas.y);
			ewl_context_menu_move(EWL_CONTEXT_MENU(eap->conmenu),x,y);
			ewl_widget_show(eap->conmenu);
			break;
		default:
			ewl_widget_hide(eap->conmenu);
			break;
	}
}
