#include"Eli_App.h"

/* internal declaration */
void 	_new_game_cb(void *data, Evas_Object *o, const char *emission, const char *source);
int 	_deal(void* data);
void 	_win_game_cb(int win, void * data);
void 	_points_cb(float points , pointsType ptype, void * data);

/* external */
void eli_app_menu_container_fill(Eli_App * eap, Evas_Object * container)
{
	const char  ** games;
	char* file;
	
	file = ecore_config_theme_with_path_from_name_get(eap->theme.gui.current);
	games = elitaire_available_games_get();
	
	for(int i = 0;games[i][0];i++){
		char* theme_name;
		Evas_Object * item;
		
		item = edje_object_add(eap->evas);
		
		if (edje_object_file_set(item, file, "element")) {
			Evas_Coord w,h;
			edje_object_size_min_get(item, NULL, &h);
			evas_object_geometry_get(container,NULL,NULL,&w,NULL);
			evas_object_resize(item, w, h);
			
			if (edje_object_part_exists(item, "elitaire_element_value")) {
				edje_object_part_text_set(item, "elitaire_element_value", games[i]);
				evas_object_show(item);

				edje_object_signal_callback_add(item, "item_selected", "item_selected", _new_game_cb, eap);
    
				esmart_container_element_append(container, item);
			} 
			else {
				fprintf( stderr, _("Elitaire Error in %s: missing element_value part!\n"),file);
				evas_object_del(item);
			}
		} 
		else {
			fprintf( stderr, _("Elitaire Error in %s: missing element group!\n"),file);
			evas_object_del(item);
		}
	}
}

void eli_app_new_game(Eli_App * eap, const char * game)
{
	Evas_Coord x,y,w,h;
	Evas_Coord min_w, min_h;
	//Evas_Object* obj;
	char* card_theme;
	int vel;
	Evas_Bool b;
	
	card_theme = ecore_config_theme_with_path_from_name_get(eap->theme.cards.current);
	
	if(eap->elitaire){ 
		evas_object_del(eap->elitaire); 
		eap->elitaire = NULL;
	}
	
	edje_object_signal_emit(eap->gui,"win,off","");
	
	/* init elitaire, set the theme, ... */
	if(!(eap->elitaire = elitaire_new(eap->evas,game))) return;
	if(elitaire_file_set(eap->elitaire,card_theme)){
		elitaire_size_min_get(eap->elitaire,&w,&h);
	}
	else {
		fprintf( stderr, _("Elitaire Error: cards file %s not found!\n"), 
			eap->theme.cards.current);
		return;
	}
	
	/* find and set the minimum size */
	elitaire_size_min_get(eap->elitaire, &min_w, &min_h);
	if(!edje_object_part_exists(eap->gui,"card_field")) {
		fprintf( stderr, _("Elitaire Error in %s: card_field does not exist!\n"),eap->theme.gui.current);
		return;
	}
	edje_object_part_geometry_get(eap->gui, "card_field" , &x,&y,&w,&h);
	if( w < min_w || h < min_h){
		Evas_Coord  win_h, win_w;
		ecore_evas_geometry_get(eap->ee, NULL, NULL, &win_w, &win_h);
		ecore_evas_resize(eap->ee, win_w + min_w - w, win_h + min_h - h);		
	}
	/* move and resize */
	evas_object_move(eap->elitaire,x,y);
	evas_object_resize(eap->elitaire, w, h);
	evas_object_show(eap->elitaire);
	
	/* tell elitaire the ecore_config vlaues */
	vel = ecore_config_int_get("velocity");
	elitaire_velocity_set(eap->elitaire,vel);
	
	vel = ecore_config_int_get("frame_rate");
	elitaire_frame_rate_set( eap->elitaire, vel);
	
	b = ecore_config_boolean_get("/graphic/animations");
	elitaire_animations_set(eap->elitaire,b);
	
	b = ecore_config_boolean_get("/graphic/shadows");
	elitaire_shadows_set(eap->elitaire,b);
	
	/* swallow the elitaire*/
	edje_object_part_swallow(eap->gui,"card_field",eap->elitaire);
	/* Add the callbacks */
	elitaire_win_callback_add(eap->elitaire,_win_game_cb,eap);
	elitaire_points_callback_add(eap->elitaire,_points_cb,eap);
	/* give evas a little bit time before starting to deal */
	ecore_timer_add(0.01,_deal,eap);
}

void eli_app_velocity_set(Eli_App *eap, int velocity)
{
	double v = 0;
	Evas_Object* obj;
	
	v = eli_app_velocity_real_to_percent(velocity);
	
	if(obj = edje_object_part_swallow_get(eap->gui,"elitaire_menu_bar")){
				
		edje_object_part_drag_value_set(obj,"elitaire_velocity_slider", v, 0);
		
		if(edje_object_part_exists(obj,"elitaire_velocity_value")){
			char buffer[5];
			
			snprintf(buffer,5,"%i",velocity);
			edje_object_part_text_set(obj,"elitaire_velocity_value",buffer);
		}
	}
	
	

}

int eli_app_velocity_get(Eli_App * eap)
{
	double v = 0;
	Evas_Object* obj;
	
	if(obj = edje_object_part_swallow_get(eap->gui,"elitaire_menu_bar")){
				
		edje_object_part_drag_value_get(obj,"elitaire_velocity_slider",&v,NULL);
	}

	return eli_app_velocity_percent_to_real(v);
}

/* internal */
int _deal(void* data)
{
	Eli_App* eap;
	eap = (Eli_App*) data;
	
	elitaire_deal(eap->elitaire);
	return 0;
}

/* to win not window */
void _win_game_cb(int win, void * data)
{
	Eli_App * eap;
	eap = (Eli_App*) data;
	if(eap){
		if(edje_object_part_exists(eap->gui,"elitaire_win_text")){			
			edje_object_part_text_set(eap->gui,"elitaire_win_text",_("You win!"));
		}
		edje_object_signal_emit(eap->gui,"win,on","");
	}
}

void _points_cb(float points , pointsType ptype, void * data)
{
	Eli_App * eap;
	eap = (Eli_App*) data;
	if(eap){
		if(edje_object_part_exists(eap->gui,"elitaire_points")){			
			char buffer[15];
			snprintf(buffer,15, _("Points: %'.f"), points);
			edje_object_part_text_set(eap->gui,"elitaire_points",buffer);
		}
	}
}

void _new_game_cb(void *data, Evas_Object *o, const char *emission, const char *source)
{
	Eli_App* eap;
	const char * game;
	
	game = edje_object_part_text_get(o,"elitaire_element_value");
	eap = (Eli_App*) data;
	
	eli_app_new_game(eap, game);
}
