#include<iostream>
#include<vector>
#include<stack>
#include<algorithm>

using namespace std;

enum stackLook{
	LAST,	//show only the last card
	ALL,	//show one below the other
	STACK,	//show it like a stack of cards
	ENDL,	//End of Line
	SEPERATOR 
};

enum pointsType{
	INTEGER,
	FLOAT,
	MONEY
};

int random_range(int n);

class cardStack;
class cardDeck;

// A Class for every kind of a playing card
class playingCard
{
	friend class cardDeck;
	
	short name;	// each card has it's own Number e.g. As = 0, 2 = 1 ...
	bool visible;
	cardDeck *myDeck;
	
	short setName(short iname) {return name = iname;}
	
	public:
	
	cardStack *myStack;
	void* data;
	
	playingCard(short iname = 0, bool iVisible = true, cardDeck* iDeck = NULL)
	{
		name = iname;
		visible = iVisible;
		myDeck = iDeck;
	}
	char  getNumValues();
	short getName() {return name;}
	short getValue() {return name % getNumValues();}
	short getSuit() {return name / getNumValues();}
	bool  isVisible() {return visible;}
	bool  setVisible(bool iVisible = true) {return visible = iVisible;}	
	
};


// the unvisble deck
class cardDeck
{
	char numSuits; // Pik 0, Herz 1, Kreuz 2, Karo 3
	char numValues;
	vector<playingCard> cards;
	vector<playingCard*> pcards;
	
	public:
		
	cardDeck(char iValues = 13, char iSuits = 4, bool iVisible = true)
	{
		numSuits = iSuits;
		numValues = iValues;
		
		short numCards = numValues * numSuits;
		
		for(short i = 0; i != numCards ; i++)
		{
			cards.push_back(playingCard(i,iVisible,this));
		}			
		for(short i = 0; i != numCards ; i++)
		{
			pcards.push_back(&cards[i]);
		}	
	}
	
	char getNumValues() {return numValues;}
	char getNumCards() {return getNumValues() * getNumSuits();}
	char getNumSuits() {return numSuits;}
	playingCard* getCard(short position) {return pcards[position];}
	playingCard* getCardByName(short cn) {return &cards[cn];}
	void shuffle() {random_shuffle(pcards.begin(), pcards.end(),random_range);}
	
	vector<playingCard*>::const_iterator begin(){
		return pcards.begin();
	}
	vector<playingCard*>::const_iterator end(){
		return pcards.end();
	}
	
};


/* ********************************************************************************************** */
/* The Stacks            								       */
/* ********************************************************************************************** */

// abstract class for any kind of stack
class cardStack
{
	protected:
	
	vector<playingCard*> cards;
	
	public:
	
	virtual ~cardStack() {}
	
	bool 		acceptCard(playingCard *pcard){
				if(empty()) return ruleEmpty(pcard);
				return rule(getLastCard(),pcard) && getLastCard()->isVisible();
				
	}
	playingCard* 	getCard(short position){return cards[position];}
	short 		getCardName(short position){
				if(!getCard(position)->isVisible()) return -1;
				else  return getCard(position)->getName();
			}
	playingCard* 	getLastCard(short invertPos=0){
				return cards[getNumCards()-invertPos-1];}
	short 		getLastCardName(short invertPos=0){
				if(!getLastCard(invertPos)->isVisible()) return -1;
				else  return getLastCard(invertPos)->getName();
			}
	short 		getNumCards() {return cards.size();}
	bool		empty() {return cards.empty();}
	short		add(playingCard *pcard){
				cards.push_back(pcard);
				pcard->myStack = this;
				return getNumCards() -1;
	}
	playingCard*	takeCard(){
				playingCard* pcard = getLastCard();
				cards.pop_back();
				return pcard;
	}
	bool 		clickOn(){
				if(empty()) return false;
				if(getLastCard()->isVisible()) return false;
				return getLastCard()->setVisible();
	}
	bool		unClickOn(){
				if(empty()) return false;
				if(!getLastCard()->isVisible()) return false;
				return getLastCard()->setVisible(false);
	}
	vector<playingCard*>::const_iterator begin(){return cards.begin();}
	vector<playingCard*>::const_iterator end()  {return cards.end();}
	int		moveable(playingCard* pcard){	//returns 0 if not moveable or  how many cards to move
				vector<playingCard*>::const_iterator it = cards.begin();
				vector<playingCard*>::const_iterator it_end = cards.end();
				int findcard = 0;
				
				while(it != it_end){
					if(findcard) { 
						if( !rule(*(it-1),*it) )return false;
						findcard++;
					}
					if( *it == pcard) findcard = 1;
					it++;
				}
				return findcard;
	}
	virtual bool	rule(playingCard* lower, playingCard* upper)=0;
	virtual bool	ruleEmpty(playingCard* pcard)=0;
	
};

// This stack only accept cards  with the same suit
class sameSuitStack : public cardStack
{	
	short startWith;
	short step;
	
	public:
		
	sameSuitStack(short iStep = 1, short iStartWith = 0){
			step = iStep;
			startWith = iStartWith;
	}
	
	virtual bool	rule(playingCard* lower, playingCard* upper){
				return lower->getSuit() == upper->getSuit() && lower->getValue() + step == upper->getValue();
	}
	virtual bool	ruleEmpty(playingCard* pcard){
				return pcard->getValue() == startWith || startWith == -1;
	}
};

class allSuitStack : public cardStack
{	
	short startWith;
	short step;
	
	public:
		
	allSuitStack(short iStep = 1, short iStartWith = 0){
			step = iStep;
			startWith = iStartWith;
	}
	
	virtual bool	rule(playingCard* lower, playingCard* upper){
				return lower->getValue() + step == upper->getValue();
	}
	virtual bool	ruleEmpty(playingCard* pcard){
				return pcard->getValue() == startWith || startWith == -1;
	}
};

class contraryColorStack : public cardStack
{
	short startWith;
	short step;		
	
	public:
	
	contraryColorStack(short iStep = -1, short iStartWith = 12){
			step = iStep;
			startWith = iStartWith;
	}
	virtual bool	rule(playingCard* lower, playingCard* upper){
				return lower->getSuit() % 2 != upper->getSuit() % 2 && lower->getValue() + step == upper->getValue();
	}
	virtual bool	ruleEmpty(playingCard* pcard){
				return pcard->getValue() == startWith || startWith == -1;
	}
};

class singleCardStack : public cardStack
{		
	public:

	virtual bool rule(playingCard* lower, playingCard* upper) {return false;}
	virtual bool ruleEmpty(playingCard* pcard) {return true;}
};

class closedStack : public cardStack
{
	public:
		
	virtual bool rule(playingCard* lower, playingCard* upper) {return false;}
	virtual bool ruleEmpty(playingCard* pcard) {return false;}
};

/* ********************************************************************************************** */
/* The Undo Classes								       */
/* ********************************************************************************************** */
class cardGame;
	
class unDoJob
{
	protected:
	float points;
	
	public:
	unDoJob(float iPoints){
		points = iPoints;
	}
	virtual void undo(cardGame* game)=0;
};

class unDoClickOn : public unDoJob
{
	protected:
	cardStack* clickOn;
	
	public:
	unDoClickOn(float iPoints, cardStack* iClickOn) : unDoJob(iPoints) {
		clickOn = iClickOn;
	}
	virtual void undo(cardGame* game);
};

class unDoMove : public unDoJob
{
	protected:
	playingCard* 	pcard;
	cardStack*	fromStack;
	
	public:
	unDoMove(float iPoints, playingCard* card, cardStack* stack) : unDoJob(iPoints) {
		pcard = card;
		fromStack = stack;
	}
	virtual void undo(cardGame* game);
};

/* ********************************************************************************************** */
/* The Games									       */
/* ********************************************************************************************** */

class cardGame
{
	friend class unDoMove; friend class unDoClickOn;
	protected:
	vector<cardStack*> stacks;
	vector<stackLook> looks;
	stack<unDoJob*> undos;
	float points;
	pointsType pType;
	
	void (*move_card)(playingCard* pcard);
	void (*flip_card)(playingCard* pcard);
	void (*win)(int win, void* data);
	void (*points_cb)(float,pointsType,void*);
	void * winData;
	void * pointsData;
	
	float addPoints(float mod){
		points += mod;
		if(points_cb) (*points_cb)(points, pType, pointsData);
		return points;
	}
	
	void moveAllUnder(playingCard* pcard, cardStack* tostack);
	
	virtual void unClickOn(cardStack* clstack){
		bool state = clstack->unClickOn();
		if( !state && flip_card){
			(*flip_card)(clstack->getLastCard());
		}
	}
	
	public:
		cardGame(){
			move_card = NULL;
			flip_card = NULL;
			win = NULL;
			winData = NULL;
			points_cb = NULL;
			points = 0;
		}
			
	virtual ~cardGame(){
		while(!stacks.empty())
		{
			delete stacks.back();
			stacks.pop_back();
		}
		while(!undos.empty())
		{
			unDoJob * job;
			job = undos.top();
			undos.pop();
			delete job;
		}
	}
	virtual bool deal()=0;
	vector<playingCard*>::const_iterator getStackBegin(int number){
		return stacks[number]->begin();}
	vector<playingCard*>::const_iterator getStackEnd(int number){
		return stacks[number]->end();}
	vector<playingCard*>::const_iterator getStackBegin(playingCard* pcard){
		return pcard->myStack->begin();}
	vector<playingCard*>::const_iterator getStackEnd(playingCard* pcard){
		return pcard->myStack->end();}
		
	playingCard* getLastCard(int number, short inverPos=0){
		return stacks[number]->getLastCard(inverPos);
	}
	char getLastCardName(int number, short pos=0){
		return stacks[number]->getLastCardName(pos);
	}
	
	vector<stackLook>::const_iterator getStackLooksBegin() {return looks.begin();}
	vector<stackLook>::const_iterator getStackLooksEnd() {return looks.end();}
	int moveableCard(playingCard* pcard){
		if(pcard->isVisible()) return pcard->myStack->moveable(pcard);
		else return 0;
	}
	virtual bool moveable(playingCard* pcard,cardStack* tostack)=0;
	bool moveable(playingCard* pcard,int tostack){
		return moveable(pcard,stacks[tostack]);
	}
	
	virtual bool move(playingCard* pcard,cardStack* tostack)=0;
	bool move(playingCard* pcard,int tostack){
		return move(pcard,stacks[tostack]);
	}	
	virtual playingCard* getCard(short name,int deck=0)=0;
	virtual bool clickOn(cardStack* clstack){
		bool state = clstack->clickOn();
		if( state && flip_card){
			if(flip_card) (*flip_card)(clstack->getLastCard());
		}
		if(state) undos.push(new unDoClickOn(0.0,clstack));
		return state;
	}
	bool clickOn(int number){
		return clickOn(stacks[number]);
	}
	bool clickOn(playingCard* pcard){
		return clickOn(pcard->myStack);
	}
	virtual int getNumCards()=0;
	int getNumStacks(){
		return stacks.size();
	}
	
	virtual bool rightClickOn(playingCard * pcard)=0;
	
	void addFlipCallback(void (*func)(playingCard* pcard)){
		flip_card = func;
	}
	
	void addMoveCallback(void (*func)(playingCard* pcard)){
		move_card = func;
	}
	void addWinCallback(void (*func)(int,void*), void * data){
		win = func;
		winData = data;
	}
	void addPointsCallback(void (*func)(float,pointsType,void*), void * data){
		points_cb = func;
		pointsData = data;
		if(func) (*func)(points, pType, data);
	}
	float getPoints() { return points;}
	pointsType getPointsType() { return pType;}
	bool undo(){
		if(undos.empty()) return false;
		else {
			unDoJob * job;
			job = undos.top();
			job->undo(this);
			undos.pop();
			delete job;
			return true;
		}
	}
};


/* freecell **********************************************************************************************/

class freeCell : public cardGame 	
{
	protected:
	cardDeck cDeck;
	
	int freeSCS(){
		int freescs = 0;
		for(int i = 0;i <= 3;i++) freescs += stacks[i]->empty();
		return freescs;
	}
	
	void checkForWin(){
		int cards = 0;
		for(int i = 4;i <= 7;i++) cards += stacks[i]->end() - stacks[i]->begin();
		
		if( cards == getNumCards() && win) (*win)(true,winData);
	}
	
	public:
	
	virtual bool deal();
	
	freeCell() : cDeck(13,4,true)
	{
		
		int i;
		
		for(i=1;i <= 4;i++) {
			looks.push_back(LAST);
			stacks.push_back(new singleCardStack());
		}
		looks.push_back(SEPERATOR);
		for(i=1;i <= 4;i++){
			looks.push_back(LAST);
			stacks.push_back(new sameSuitStack());
		}
		looks.push_back(ENDL);
		for(i=1;i <= 8;i++){
			looks.push_back(ALL);
			stacks.push_back(new contraryColorStack(-1,-1));
		}
		
		cDeck.shuffle();
	}
	
	virtual ~freeCell(){
	}
	
	virtual bool moveable(playingCard* pcard,cardStack* tostack){
		if(!(moveableCard(pcard) - freeSCS() <= 1 && tostack->acceptCard(pcard) && moveableCard(pcard)))
			return false;
		
		vector<playingCard*>::const_iterator it 	= pcard->myStack->begin();
		vector<playingCard*>::const_iterator it_end 	= pcard->myStack->end();
		
		while(*it != pcard) it++;
		it++;
		if(it == it_end) return true; 
		else return tostack->rule(pcard,*it);
	}
	virtual bool move(playingCard* pcard, cardStack* tostack){
		if(!moveable(pcard,tostack)) return false;
		else{
			undos.push(new unDoMove(1.0,pcard,pcard->myStack));
			addPoints(1.0);
			moveAllUnder(pcard,tostack);
			checkForWin();
			return true;
		}
	}
	virtual playingCard* getCard(short name,int pos=0){
		return cDeck.getCardByName(name);
	}
	virtual int getNumCards(){
		return cDeck.getNumCards();
	}
	
	virtual bool rightClickOn(playingCard * pcard){
		for(int i = 4; i < 8; i++){
			if(move(pcard,stacks[i])){
				//if(move_card) (*move_card)(pcard);
				return true;
			}
		}
		for(int i = 0; i < 4; i++){
			if(move(pcard,stacks[i])){
				//if(move_card) (*move_card)(pcard);
				return true;
			}
		}
		return false;
	}
};

/* klondike **********************************************************************************************/
class klondike : public cardGame
{
	protected:
	
	bool isThoughtful;
	
	cardDeck cDeck;
	bool clickOn0();
	void unClickOn0();
	
	void checkForWin(){
		int cards = 0;
		for(int i = 2;i <= 5;i++) cards += stacks[i]->end() - stacks[i]->begin();
		if( cards == getNumCards() && win) (*win)(true,winData);
	}
	
	virtual void unClickOn(cardStack* clstack){
		if(clstack == stacks[0]) unClickOn0();
		else cardGame::unClickOn(clstack);			
	}
	
	public:
	
	virtual bool deal();
	
	klondike(bool thoughtful = false) : cDeck(13,4,false), isThoughtful(thoughtful)
	{		
		
		int i = 0;
		for(i=1;i <= 2;i++){
			looks.push_back(STACK);
			stacks.push_back(new closedStack());
		}
		looks.push_back(SEPERATOR);
		for(i=1;i <= 4;i++){
			looks.push_back(LAST);
			stacks.push_back(new sameSuitStack());
		}
		looks.push_back(ENDL);
		for(i=1;i <= 7;i++){
			looks.push_back(ALL);
			stacks.push_back(new contraryColorStack());
		}
		
		cDeck.shuffle();
		//deal();
		
	}
	
	virtual ~klondike(){
	}
	
	virtual bool moveable(playingCard* pcard,cardStack* tostack){
		if(!(tostack->acceptCard(pcard) && moveableCard(pcard)))
			return false;
		
		vector<playingCard*>::const_iterator it 	= pcard->myStack->begin();
		vector<playingCard*>::const_iterator it_end 	= pcard->myStack->end();
		
		while(*it != pcard) it++;
		it++;
		if(it == it_end) return true; 
		else return tostack->rule(pcard,*it);
		
	}
	
	virtual bool move(playingCard* pcard,cardStack* tostack){
		if(!moveable(pcard,tostack)) return false;
		else {
			undos.push(new unDoMove(1.0,pcard,pcard->myStack));
			moveAllUnder(pcard,tostack);
			addPoints(1.0);
			checkForWin();
		}
		return true;
	}
	virtual playingCard* getCard(short name,int pos=0){
		return cDeck.getCardByName(name);
	}
	virtual bool clickOn(cardStack* clstack){
		if(clstack == stacks[0]) return clickOn0();
		else return cardGame::clickOn(clstack);			
	}
	virtual bool clickOn(int number){
		return clickOn(stacks[number]);
	}
	virtual bool clickOn(playingCard* pcard){
		return clickOn(pcard->myStack);
	}
	virtual int getNumCards(){
		return cDeck.getNumCards();
	}
	virtual bool rightClickOn(playingCard * pcard){
		for(int i = 2; i < 6; i++){
			if(move(pcard,stacks[i])){
				//if(move_card) (*move_card)(pcard);
				return true;
			}
		}
		return false;
	}
};

/* Baker's Dozen *******************************************************************************************/

class bakersDozen : public cardGame
{
	protected:
	
	cardDeck cDeck;
	
	void checkForWin(){
		int cards = 0;
		for(int i = 4;i <= 8;i++) cards += stacks[i]->end() - stacks[i]->begin();
		if( cards == getNumCards() && win) (*win)(true,winData);
	}
	
	public:
	
	virtual bool deal();
	
	bakersDozen() : cDeck(13,4,true)
	{		
		
		int i;
		
		for(i=1;i <= 4;i++){
			looks.push_back(ALL);
			stacks.push_back(new allSuitStack(-1,-2));
		}
		looks.push_back(SEPERATOR);
		for(i=1;i <= 4;i++){
			looks.push_back(LAST);
			stacks.push_back(new sameSuitStack());
		}
		looks.push_back(ENDL);
		looks.push_back(ENDL);
		looks.push_back(ENDL);
		
		for(i=1;i <= 9;i++){
			looks.push_back(ALL);
			stacks.push_back(new allSuitStack(-1,-2));
		}
		
		cDeck.shuffle();
		
	}
	
	virtual ~bakersDozen(){
	}
	
	virtual bool moveable(playingCard* pcard,cardStack* tostack){
		if(tostack->acceptCard(pcard) && moveableCard(pcard) == 1)
			return true;
		else return false;
	}
	
	virtual bool move(playingCard* pcard,cardStack* tostack){
		if(!moveable(pcard,tostack)) return false;
		
		tostack->add(pcard->myStack->takeCard());
		
		checkForWin();
		return true;
	}
	virtual playingCard* getCard(short name,int pos=0){
		return cDeck.getCardByName(name);
	}
	virtual bool clickOn(cardStack* clstack){
		return false;
	}
	virtual bool clickOn(int number){
		return clickOn(stacks[number]);
	}
	virtual bool clickOn(playingCard* pcard){
		return clickOn(pcard->myStack);
	}
	virtual int getNumCards(){
		return cDeck.getNumCards();
	}	
	virtual bool rightClickOn(playingCard * pcard){
		for(int i = 4; i < 8; i++){
			if(move(pcard,stacks[i])){
				if(move_card) (*move_card)(pcard);
				return true;
			}
		}
		return false;
	}
};



/* Randomizer *********************************************************************** */
inline int random_range(int n)
{
	return (int) (double(n)*rand()/(RAND_MAX + 1.0));
}

