#include<Ecore.h>
#include<Ecore_File.h>
#include<Ecore_Config.h>
#include<stdlib.h>
#include<Eli_App.h>
//#include"Elitaire.h"

char * theme_cut_off_suffix(const char* file);
char* theme_string_cat(const char* txt1, const char* txt2);
void _cards_item_selected_cb(void* data, Evas_Object* o, const char *emission, const char *source);
void _theme_item_selected_cb(void* data, Evas_Object* o, const char *emission, const char *source);
inline Evas_Bool _eli_app_theme_exists( Ecore_List * l, const char * theme);

Evas_Bool edje_file_group_exists(const char * file, const char * group)
{
	Evas_List * collection;
	
	collection = edje_file_collection_list(file);
	if(collection){
		Evas_List *l;
		l = collection;
		
		while(l){
			if(!strcmp( (const char*) l->data, group)){
				edje_file_collection_list_free(collection);
				return true;
			}
			l = l->next;
		}
	}
	
	edje_file_collection_list_free(collection);
	return false;
}

void eli_app_theme_init(Eli_App * eap)
{
	Ecore_List* l;
	Ecore_List* gui, *cards;
	char* name;
	char* file;
	
	gui   = ecore_list_new();
	cards = ecore_list_new();
	ecore_list_init( gui );
	ecore_list_init(cards);
	
	l = eli_app_theme_names_get();
	
	/* selecting the gui theme */
	while(l->current){
		name = (char*) l->current->data;
		file = ecore_config_theme_with_path_from_name_get(name);
		
		/* selecting the gui theme */
		if(edje_file_group_exists(file,"main")) ecore_list_append(gui,name);
		/* selecting the card theme */
		if(edje_file_group_exists(file,"01")) ecore_list_append(cards,name);
		
		free(file);
		ecore_list_next(l);
	}
	l->current = l->first;

	eap->theme.gui.list   = gui;
	eap->theme.cards.list = cards;
	
	//ecore_list_clear(l);
	ecore_list_destroy(l);
}

/* return a list of all files with .edj suffix*/
Ecore_List* eli_app_theme_names_get() 
{
   
	Ecore_List* file_l, *dir_l, *out_l;
	
	dir_l = eli_app_theme_dir_get();
	
	out_l = ecore_list_new();
	ecore_list_init(out_l);
	
	while(dir_l->current)
	{
		char* dir;
		dir = (char*) dir_l->current->data;
		file_l = ecore_file_ls(dir);
		
		if(file_l){			
			while(file_l->current)
			{
				char * file;
				char * sufbegin;
				
				file = (char*) file_l->current->data;
				sufbegin = strrchr(file, '.');
				
				if(!strcmp(sufbegin,".edj"))
					ecore_list_append(out_l,theme_cut_off_suffix(file));
				ecore_list_next(file_l);
			}
			ecore_list_destroy(file_l);
		}
		ecore_list_next(dir_l);
	}
	
	ecore_list_destroy(dir_l);
	
	out_l->current = out_l->first;
	
	return out_l;
}

Ecore_List * eli_app_theme_dir_get()
{
	Ecore_List* l;
	char * dirs, *pt_dirs, * rem_dir;

	dirs = ecore_config_theme_search_path_get();
	rem_dir = pt_dirs = dirs;
	
	l = ecore_list_new();
	ecore_list_init(l);
	
	while(*pt_dirs != '\0'){
		if(*pt_dirs == '|'){
			*pt_dirs = '\0';

			ecore_list_append(l, dirs);
			dirs = pt_dirs + 1;
		}
		
		pt_dirs++;
	}
		
	ecore_list_append(l,dirs);
	ecore_list_goto_first(l);
	
	return l;
}

char * theme_cut_off_suffix(const char* file)
{
	char * new_file, * pt;
	char * end;
	
	/*find the begin of the suffix, the new end*/
	end = strrchr(file, '.');
	
	/*copy the rest*/
	new_file = (char*) malloc(sizeof(char)*(end - file + 1));
	pt = new_file;
	
	while(file != end) *pt++ = *file++;
	*pt = '\0';
	
	return new_file;
	
}

char * theme_string_cat(const char* txt1,const char* txt2)
{
	char * newtxt, *pt;
	size_t len;
	
	len = strlen(txt1) + strlen(txt2);

	newtxt = (char*) malloc(sizeof(char) * (len + 1));
	pt = newtxt;
	
	while(*txt1 != '\0')  *pt++ = *txt1++;
	while(*txt2 != '\0')  *pt++ = *txt2++;
	*pt = '\0';
	
	return newtxt;
}

/* Fill the theme container for selecting the theme */
void eli_app_theme_container_fill(Eli_App * eap, Evas_Object * container)
{
	Ecore_List * l;
	char* file;
	
	file = ecore_config_theme_with_path_from_name_get(eap->theme.gui.current);
	l = eap->theme.gui.list;
	
	while(l->current){
		char* theme_name;
		Evas_Object * item;
		theme_name = (char*) l->current->data;
		item = edje_object_add(eap->evas);
		
		if (edje_object_file_set(item, file, "element")) {
			Evas_Coord w,h;
			edje_object_size_min_get(item, NULL, &h);
			evas_object_geometry_get(container,NULL,NULL,&w,NULL);
			evas_object_resize(item, (int)w, (int)h);
			
			if (edje_object_part_exists(item, "elitaire_element_value")) {
				edje_object_part_text_set(item, "elitaire_element_value", theme_name);
				evas_object_show(item);

				edje_object_signal_callback_add(item, "item_selected", "item_selected", _theme_item_selected_cb, eap);
    
				esmart_container_element_append(container, item);
			} 
			else {
				fprintf( stderr, _("Elitaire Error in %s: missing element_value part!\n"),file);
				evas_object_del(item);
			}
		} 
		else {
			fprintf( stderr, _("Elitaire Error in %s: missing element group!\n"),file);
			evas_object_del(item);
		}
		ecore_list_next(l);
	}
	
	l->current = l->first;
	if(file) free(file);
}

/* Fill the cards container for selecting the carddeck */
void eli_app_cards_container_fill(Eli_App * eap, Evas_Object * container)
{
	Ecore_List * l;
	char* file;
	
	file = ecore_config_theme_with_path_from_name_get(eap->theme.gui.current);
	l = eap->theme.cards.list;
	
	
	
	while(l->current){
		char* cards_name;
		Evas_Object * item;
		
		cards_name = (char*) l->current->data;
		item = edje_object_add(eap->evas);
		
		if (edje_object_file_set(item, file, "element")) {
			Evas_Coord w,h;
			edje_object_size_min_get(item, NULL, &h);
			evas_object_geometry_get(container,NULL,NULL,&w,NULL);
			evas_object_resize(item, (int)w, (int)h);
			
			if (edje_object_part_exists(item, "elitaire_element_value")) {
				edje_object_part_text_set(item, "elitaire_element_value", cards_name);
				evas_object_show(item);

				edje_object_signal_callback_add(item, "item_selected", "item_selected", _cards_item_selected_cb, eap);
    
				esmart_container_element_append(container, item);
			} 
			else {
				fprintf( stderr, _("Elitaire Error in %s: missing element_value part!\n"),file);
				evas_object_del(item);
			}
		} 
		else {
			fprintf( stderr, _("Elitaire Error in %s: missing element group!\n"),file);
			evas_object_del(item);
		}
		ecore_list_next(l);
	}
	
	l->current = l->first;
	if(file) free(file);
}

void eli_app_theme_change(Eli_App* eap, char * theme)
{
	if(eap->theme.gui.current) free(eap->theme.gui.current);
	eap->theme.gui.current = theme;
	eli_app_gui_del(eap);
	eli_app_gui_make(eap);
}

Evas_Bool eli_app_theme_gui_exists(Eli_App * eap, const char * theme)
{
	return _eli_app_theme_exists( eap->theme.gui.list, theme);
}

Evas_Bool eli_app_theme_cards_exists(Eli_App * eap, const char * theme)
{
	return _eli_app_theme_exists( eap->theme.cards.list, theme);
}

inline Evas_Bool _eli_app_theme_exists( Ecore_List * l, const char * theme)
{
	if(!l) return false;
	
	l->current = l->first;
	while(l->current){
		char * data;
		
		data = (char*) l->current->data;
		if(!strcmp(theme,data)) {
			l->current = l->first;
			return true;
		}
		ecore_list_next(l);
	}
	l->current = l->first;
	
	return false;
}

void _theme_item_selected_cb(void* data, Evas_Object* o, const char *emission, const char *source)
{
	char* selected;
	selected = strdup(edje_object_part_text_get(o,"elitaire_element_value"));
	ecore_config_theme_set("/theme/gui",selected);
	free(selected);
}

void _cards_item_selected_cb(void* data, Evas_Object* o, const char *emission, const char *source)
{
	char* selected;
	selected = strdup(edje_object_part_text_get(o,"elitaire_element_value"));
	ecore_config_theme_set("/theme/cards",selected);
	free(selected);
}
